<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Admin form dataset.
 *
 * @package HostCMS 6\Admin
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
abstract class Admin_Form_Dataset
{
	protected $_limit = NULL;
	protected $_offset = 0;

	protected $_loaded = FALSE;

	protected $_changedFields = array();

	protected $_Admin_Form_Controller = NULL;

	public function controller(Admin_Form_Controller $controller)
	{
		$this->_Admin_Form_Controller = $controller;
		return $this;
	}
	
	/**
	 * Array of conditions
	 * array (
	 * 	array('where' => array('a', '=', 'b')),
	 * 	array('where' => array('c', '=', 'd'))
	 * )
	 */
	protected $_conditions = array();

	public function limit($limit)
	{
		$this->_limit = intval($limit);
		return $this;
	}

	public function offset($offset)
	{
		$this->_offset = intval($offset);
		return $this;
	}

	public function loaded($loaded)
	{
		$this->_loaded = $loaded;
		return $this;
	}

	public function addCondition($condition)
	{
		$this->_conditions[] = $condition;
		return $this;
	}

	// Изменение данных поля Admin_Form_Field для источника
	// используется, например, для изменение типа выводимого поля
	public function changeField($name, $property, $value)
	{
		$this->_changedFields[$name][$property] = $value;
		return $this;
	}

	public function getFieldChanges($name)
	{
		if (isset($this->_changedFields[$name]))
		{
			return $this->_changedFields[$name];
		}

		return NULL;
	}

	abstract public function getCount();

	abstract public function load();

	abstract public function getEntity();
	
	static public function _sortAsc($m, $n)
	{
		$sortField = $m->getSortField();
		if ($m->$sortField == $n->$sortField)
		{
			return 0;
		}

		return ($m->$sortField < $n->$sortField) ? -1 : 1;
	}

	static public function _sortDesc($m, $n)
	{
		$sortField = $m->getSortField();
		if ($m->$sortField == $n->$sortField)
		{
			return 0;
		}

		return ($m->$sortField > $n->$sortField) ? -1 : 1;
	}
}