<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Default entity controller
 *
 * @package HostCMS 6\Core
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
class Core_Controller extends Core_Servant_Properties
{
	/**
	 * Allowed object properties
	 * @var array
	 */
	protected $_allowedProperties = array();

	protected $_entity = NULL;
	protected $_xsl = NULL;

	/**
	 * Constructor.
	 */
	public function __construct(Core_Entity $oEntity)
	{
		$this->setEntity($oEntity);
		parent::__construct();
	}

	protected $_entities = array();

	public function addEntity($oChildrenEntity)
	{
		$this->_entities[] = $oChildrenEntity;
		return $this;
	}

	public function setEntity(Core_Entity $entity)
	{
		$this->_entity = $entity;
		return $this;
	}

	public function getEntity()
	{
		return $this->_entity;
	}

	public function addEntities(array $aChildrenEntities)
	{
		foreach ($aChildrenEntities as $oEntity)
		{
			$this->addEntity($oEntity);
		}

		return $this;
	}

	public function clearEntities()
	{
		$this->_entities = array();
		return $this;
	}

	public function xsl(Xsl_Model $oXsl)
	{
		$this->_xsl = $oXsl;
		return $this;
	}

	/**
	 * Get XML for entity and children entities
	 * @return string
	 */
	public function getXml()
	{
		$this->_entity
			->clearEntities()
			->addEntities($this->_entities);

		return $this->_entity->getXml();
	}

	/**
	 * Show built data
	 * @see get()
	 * @return Core_Controller
	 */
	public function show()
	{
		echo $this->get();
		return $this;
	}

	/**
	 * Get HTML based by entities tree, use XSL $this->_xsl
	 * @see getXml()
	 * @return string
	 */
	public function get()
	{
		if (is_null($this->_xsl))
		{
			throw new Core_Exception('Xsl does not exist.');
		}

		$sXml = $this->getXml();

		$return = Xsl_Processor::instance()
			->xml($sXml)
			->xsl($this->_xsl)
			->process();

		$this->clearEntities();

		/*echo "<br />Build HTML from XML and XSL '{$this->_xsl->name}'",
			"<pre>" . htmlspecialchars(
				Xsl_Processor::instance()->formatXml($sXml)
			) . "</pre>";*/

		return $return;
	}

	public function __toString()
    {
		$str = parent::__toString();
		!is_null($this->_xsl) && $str .= ',xsl' . '=' . $this->_xsl->name;

		return $str;
    }
}
