<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Abstract HTTP
 *
 * @package HostCMS 6\Core\Http
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
abstract class Core_Http
{
	static protected function _getDriverName($driver)
	{
		return __CLASS__ . '_' . ucfirst($driver);
	}

	/**
	 * Register an existing instance as a singleton.
	 * @param string $name 
	 * @return object
	 */
	static public function instance($name = 'default')
	{
		if (!is_string($name))
		{
			throw new Core_Exception('Wrong argument type (expected String)');
		}

		$aConfig = Core::$config->get('core_http');

		if (!isset($aConfig[$name]) || !isset($aConfig[$name]['driver']))
		{
			throw new Core_Exception('Core_Http "%name" configuration doesn\'t defined', array('%name' => $name));
		}

		$driver = self::_getDriverName($aConfig[$name]['driver']);
		$oDriver = new $driver($aConfig[$name]);

		return $oDriver->config($aConfig[$name]);
	}

	protected $_config = array();
	public function config($array)
	{
		$this->_config = $array;
		return $this;
	}

	abstract protected function _execute($host, $path, $query);

	/**
	 * Constructor.
	 */
	public function __construct()
	{
		$this->_init();
	}

	protected function _init()
	{
		$this
			->userAgent('Mozilla/5.0 (compatible; HostCMS/6.x; +http://www.hostcms.ru)')
			->method('GET')
			->timeout(10)
			->port(80)
			->contentType('application/x-www-form-urlencoded');

		return $this;
	}

	protected $_userAgent = NULL;
	public function userAgent($userAgent)
	{
		$this->_userAgent = $userAgent;
		return $this;
	}

	protected $_url = NULL;
	public function url($url)
	{
		$this->_url = $url;
		return $this;
	}

	protected $_referer = NULL;
	public function referer($referer)
	{
		$this->_referer = $referer;
		return $this;
	}

	protected $_method = NULL;
	public function method($method)
	{
		$this->_method = $method;
		return $this;
	}

	protected $_timeout = NULL;
	public function timeout($timeout)
	{
		$this->_timeout = $timeout;
		return $this;
	}

	protected $_port = NULL;
	public function port($port)
	{
		$this->_port = $port;
		return $this;
	}

	protected $_contentType = NULL;
	public function contentType($contentType)
	{
		$this->_contentType = $contentType;
		return $this;
	}

	protected $_data = array();
	public function data($key, $value)
	{
		$this->_data[$key] = $value;
		return $this;
	}

	protected $_headers = NULL;
	public function getHeaders()
	{
		return $this->_headers;
	}

	protected $_body = NULL;
	public function getBody()
	{
		return $this->_body;
	}

	/**
	 * Executes the business logic.
	 */
	public function execute()
	{
		$aUrl = @parse_url(trim($this->_url));

		$path = isset($aUrl['host']) && isset ($aUrl['path'])
			? $aUrl['path']
			: '/';

		$host = isset($aUrl['host']) ?
			$aUrl['host']
			: '';

		$query = isset ($aUrl['query']) ?
			'?' . $aUrl['query']
			: '';

		$this->_referer = is_null($this->_referer)
			? "http://{$host}"
			: $this->_referer;

		return $this->_execute($host, $path, $query);
	}
}