<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Abstract mail
 *
 * @package HostCMS 6\Core\Mail
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
abstract class Core_Mail
{
	protected $_status = FALSE;

	public function getStatus()
	{
		return $this->_status;
	}

	static protected function _getDriverName($driver)
	{
		return __CLASS__ . '_' . ucfirst($driver);
	}

	/**
	 * Register an existing instance as a singleton.
	 * @param string $name
	 * @return object
	 */
	static public function instance($name = 'default', array $aPersonalConfig = array())
	{
		if (!is_string($name))
		{
			throw new Core_Exception('Wrong argument type (expected String)');
		}

		$aConfig = Core::$config->get('core_mail');

		if (!isset($aConfig[$name]) || !isset($aConfig[$name]['driver']))
		{
			throw new Core_Exception('Core_Mail "%name" configuration doesn\'t defined', array('%name' => $name));
		}

		$driver = self::_getDriverName($aConfig[$name]['driver']);
		$oDriver = new $driver();

		return $oDriver->config($aPersonalConfig + $aConfig[$name]);
	}

	protected $_config = array();
	public function config($array)
	{
		$this->_config = $array;
		return $this;
	}

	abstract protected function _send($to, $subject, $message, $additional_headers);

	/**
	 * Constructor.
	 */
	public function __construct()
	{
		$this->bound('----------' . mb_strtoupper(uniqid(time())));
	}

	protected $_separator = "\n";
	public function separator($separator)
	{
		$this->_separator = $separator;
		return $this;
	}

	protected $_to = NULL;
	public function to($to)
	{
		$this->_to = $to;
		return $this;
	}

	protected $_from = NULL;
	public function from($from)
	{
		$this->_from = $from;
		return $this;
	}

	protected $_subject = NULL;
	public function subject($subject)
	{
		$this->_subject = $subject;
		return $this;
	}

	protected $_message = NULL;
	public function message($message)
	{
		$this->_message = $message;
		return $this;
	}

	protected $_senderName = NULL;
	public function senderName($senderName)
	{
		$this->_senderName = $senderName;
		return $this;
	}

	protected $_bound = NULL;
	public function bound($bound)
	{
		$this->_bound = $bound;
		return $this;
	}

	protected $_contentType = 'text/plain';
	public function contentType($contentType)
	{
		$this->_contentType = $contentType;
		return $this;
	}

	protected $_multipartRelated = FALSE;
	public function multipartRelated($multipartRelated)
	{
		$this->_multipartRelated = $multipartRelated;
		return $this;
	}

	protected $_headers = array();
	public function header($name, $value)
	{
		$this->_headers[$name] = $value;
		return $this;
	}

	protected $_files = array();

	/**
	 * Attach file
	 * <code>
	 * $oCore_Mail->attach(array(
	 * 'filepath' => CMS_FOLDER . 'file.jpg',
	 * 'filename' => 'file.jpg',
	 * 'Content-ID' => '123456',
	 * // attachment or inline
	 * 'Content-Disposition' => 'attachment',
	 * 'Content-Type' => 'application/octet-stream'
	 * ));
	 * </code>
	 */
	public function attach($attach)
	{
		$this->_files[] = $attach;
		return $this;
	}

	public function send()
	{
		$sFrom = !is_null($this->_senderName)
			? '=?UTF-8?B?' . base64_encode($this->_senderName)."?= <{$this->_from}>"
			: $this->_from;

		$this->header('From', $sFrom)
			->header('X-Mailer', 'HostCMS');

		if (!isset($this->_headers['Reply-To']))
		{
			$this->header('Reply-To', "<{$this->_from}>");
		}

		if (!isset($this->_headers['Return-Path']))
		{
			$this->header('Return-Path', "<{$this->_from}>");
		}

		$this
			->header('MIME-Version', '1.0')
			->header('Content-Type', "multipart/mixed; boundary=\"{$this->_bound}\"")
			;

		$sSingleSeparator = $this->_separator;
		$sDoubleSeparators = $sSingleSeparator . $sSingleSeparator;

		$content = "This is a multi-part message in MIME format.{$sDoubleSeparators}";

		// Для почтовых рассылок, чтобы картинки были внутри письма
		if (count($this->_files) > 0 && $this->_multipartRelated)
		{
			$content .= "--{$this->_bound}{$sSingleSeparator}";

			// Change bound
			$this->_bound = '---------==' . strtoupper(uniqid(time()));

			$content .= "Content-Type: Multipart/Related;";
			$content .= " boundary=\"{$this->_bound}\"";
			$content .= $sDoubleSeparators;
		}

		$content .= "--{$this->_bound}{$sSingleSeparator}";
		$content .= "Content-Type: {$this->_contentType}; charset=UTF-8{$sSingleSeparator}";
		//$content .= "Content-Transfer-Encoding: 8bit";
		$content .= "Content-Transfer-Encoding: base64";
		$content .= $sDoubleSeparators;
		//$content .= str_replace("\r", "", $this->_message);
		$content .= chunk_split(base64_encode($this->_message));
		$content .= $sDoubleSeparators;

		if (count($this->_files) > 0)
		{
			foreach ($this->_files as $value)
			{
				if (isset($value['filepath']) && isset($value['filename']) && is_file($value['filepath']))
				{
					try
					{
						$content .= "--{$this->_bound}{$sSingleSeparator}";

						$ContentType = isset($value['Content-Type'])
							? $value['Content-Type']
							: 'application/octet-stream';

						$content .= "Content-Type: {$ContentType};";
						$content .= " name=\"{$value['filename']}\"{$sSingleSeparator}";
						$content .= "Content-Transfer-Encoding: base64{$sSingleSeparator}";

						if (isset($value['Content-ID']))
						{
							$content .= "Content-ID: <{$value['Content-ID']}>{$sSingleSeparator}";
						}

						$ContentDisposition = isset($value['Content-Disposition'])
							? $value['Content-Disposition']
							: 'attachment';

						$filename = '=?UTF-8?B?' . base64_encode($value['filename']) . '?=';
						$content .= "Content-Disposition: {$ContentDisposition};";
						$content .= " filename=\"{$filename}\"";
						$content .= $sDoubleSeparators;

						$content .= chunk_split(base64_encode(Core_File::read($value['filepath'])));
					}
					catch (Exception $e){}
				}

				$content .= $sDoubleSeparators;
			}
		}

		// Final bound with --
		$content .= "--{$this->_bound}--{$sSingleSeparator}";

		$subject = '=?UTF-8?B?' . base64_encode($this->_subject) . '?=';

		$aHeaders = array();
		foreach ($this->_headers as $headerName => $headerValue)
		{
			$aHeaders[] = "{$headerName}: {$headerValue}";
		}

		$header = implode($sSingleSeparator, $aHeaders);

		return $this->_send($this->_to, $subject, $content, $header);
	}
}