<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Sessions
 *
 * @package HostCMS 6\Core
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
class Core_Session
{
	/**
	 * Lock prefix
	 * @var string
	 */
	protected $_lockPrefix = NULL;

	/**
	 * GET_LOCK timeout (sec)
	 * @var int
	 */
	protected $_getLockTimeout = 1;

	/**
	 * Next step delay (microseconds)
	 * Default 0,5 sec - 500000 microseconds
	 * @var int
	 */
	protected $_nextStepDelay = 500000;

	/**
	 * Lock timeout
	 * @var int
	 */
	protected $_lockTimeout = 10;

	protected $_dataBase = NULL;

	/**
	 * Constructor.
	 */
	public function __construct()
	{
		$this->_dataBase = Core_DataBase::instance();

		if ($this->_lockPrefix === NULL)
		{
			$aDataBaseConfig = $this->_dataBase->getConfig();
			$this->_lockPrefix = $aDataBaseConfig['database'] . '_' . 'sessions';
		}
	}

	static protected $_started = FALSE;

	static public function start()
	{
		if (!self::$_started)
		{
			$oCore_Session = new self();
			session_set_save_handler(
				array($oCore_Session, 'sessionOpen'),
				array($oCore_Session, 'sessionClose'),
				array($oCore_Session, 'sessionRead'),
				array($oCore_Session, 'sessionWrite'),
				array($oCore_Session, 'sessionDestroyer'),
				array($oCore_Session, 'sessionGc')
			);

			// При повторном запуске $_SESSION уже будет
			//if (Core_Array::getRequest(session_name())/* && !isset($_SESSION)*/)
			//{
				@session_start();
				self::$_started = TRUE;
			//}

			$domain = Core_Array::get($_SERVER, 'HTTP_HOST');
			if (!empty($domain))
			{
				// Обрезаем www у домена
				strpos($domain, 'www.') === 0 && $domain = substr($domain, 4);

				// Явное указание domain возможно только для домена второго и более уровня
				// http://wp.netscape.com/newsref/std/cookie_spec.html
				// http://web-notes.ru/2008/07/cookies_within_local_domains/
				$domain = strpos($domain, '.') !== FALSE && !Core_Valid::ip($domain)
					? '.' . $domain
					: '';

				setcookie(session_name(), session_id(), time() + 31536000, '/', $domain);
			}
		}
		return TRUE;
	}

	static public function close()
	{
		//if (self::$_started)
		//{
			self::$_started = FALSE;
			session_write_close();
		//}
		return FALSE;
	}

	public function sessionOpen($save_path, $session_name)
	{
		return TRUE;
	}

	public function sessionClose()
	{
		return TRUE;
	}

	protected function _lock($id)
	{
		$iStartTime = time();

		while (TRUE)
		{
			$oDataBase = $this->_dataBase->setQueryType(0)
				->query('SELECT GET_LOCK("' . $this->_dataBase->quote($this->_lockPrefix . '_' . $id) . '",'
				. intval($this->_getLockTimeout) . ') AS `lock`');

			$row = $oDataBase->asAssoc()->current();

			if (!is_array($row))
			{
				exit('HostCMS session lock error: Get row failure');
			}

			if (isset($row['lock']) && $row['lock'] == 1)
			{
				// Блокировка удалась
				break;
			}

			// Время, прошедшее с начала попытки блокировки
			$iTime = time() - $iStartTime;

			if ($iTime > $this->_lockTimeout)
			{
				exit('HostCMS session lock error: Timeout');
			}

			usleep($this->_nextStepDelay);
		}

		return TRUE;
	}

	protected function _unlock($id)
	{
		$oDataBase = $this->_dataBase->setQueryType(0)
			->query('SELECT RELEASE_LOCK("' . $this->_dataBase->quote($this->_lockPrefix . '_' . $id) . '") AS `lock`');

		$row = $oDataBase->asAssoc()->current();

		if (!is_array($row))
		{
			exit('HostCMS session unlock error: Get row failure');
		}

		return TRUE;
	}

	public function sessionRead($id)
	{
		$this->_lock($id);

		$queryBuilder = Core_QueryBuilder::select('value')
			->from('sessions')
			->where('id', '=', $id)
			->limit(1);

		$row = $queryBuilder->execute()->asAssoc()->current();

		if ($row)
		{
			// Update last change time
			Core_QueryBuilder::update('sessions')
				//->columns(array('time' => 'UNIX_TIMESTAMP(NOW())'))
				->columns(array('time' => time()))
				->where('id', '=', $id)
				->execute()
				;

			return $row['value'];
		}

		return FALSE;
	}

	public function sessionWrite($id, $value)
	{
		$this->_lock($id);

		$oDataBase = Core_QueryBuilder::update('sessions')
				//->columns(array('time' => 'UNIX_TIMESTAMP(NOW())'))
				->set('value', $value)
				->set('time', time())
				->where('id', '=', $id)
				->execute();

		if ($oDataBase->getAffectedRows() == 0)
		{
			$maxlifetime = intval(ini_get('session.gc_maxlifetime'));

			Core_QueryBuilder::insert('sessions')
				->ignore()
				->columns('id', 'value', 'time', 'maxlifetime')
				->values($id, $value, time(), $maxlifetime)
				->execute();
		}

		$this->_unlock($id);

		return TRUE;
	}

	public function sessionDestroyer($id)
	{
		$this->_lock($id);

		Core_QueryBuilder::delete('sessions')
			->where('id', '=', $id)
			->execute();

		// для предотвращения автоматической повторной регистрации сеанса
		$_SESSION = array();

		return TRUE;
	}

	public function sessionGc($maxlifetime)
	{
		Core_QueryBuilder::delete('sessions')
			->where('time + maxlifetime', '<', time())
			->execute();

		return TRUE;
	}
}