<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Google sitemap
 * http://www.sitemaps.org/protocol.html
 *
 * @package HostCMS 6\Core
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
class Core_Sitemap
{
	protected $_oSite = NULL;
	public function __construct(Site_Model $oSite)
	{
		$this->_oSite = $oSite;
	}

	protected $_createIndex = FALSE;
	public function createIndex($createIndex)
	{
		$this->_createIndex = $createIndex;
		return $this;
	}

	protected $_perFile = NULL;
	public function perFile($perFile)
	{
		$this->_perFile = intval($perFile);
		return $this;
	}

	protected $_aIndexedFiles = array();

	protected $_currentOut = NULL;
	protected function _getOut()
	{
		if ($this->_createIndex)
		{
			if (is_null($this->_currentOut) || $this->_inFile > $this->_perFile)
			{
				$this->_getNewOutFile();
			}
		}
		elseif (is_null($this->_currentOut))
		{
			$this->_currentOut = new Core_Out_Std();
			$this->_open();
		}

		return $this->_currentOut;
	}

	protected $_inFile = 0;
	protected $_countFile = 0;

	protected function _open()
	{
		$this->_currentOut->open();
		$this->_currentOut->write('<?xml version="1.0" encoding="UTF-8"?>' . "\n")
			->write('<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' . "\n");
		return $this;
	}

	protected function _close()
	{
		$this->_currentOut->write("</urlset>\n");
		$this->_currentOut->close();
		return $this;
	}

	protected function _getNewOutFile()
	{
		if (!is_null($this->_currentOut))
		{
			$this->_currentOut->close();

			$this->_countFile++;
			$this->_inFile = 0;
		}

		$this->_aIndexedFiles[] = $filename = "sitemap{$this->_countFile}.xml";

		$this->_currentOut = new Core_Out_File();
		$this->_currentOut->filePath(CMS_FOLDER . $filename);
		$this->_open();
	}

	public function addNode($loc, $changefreq, $priority)
	{
		switch ($changefreq)
		{
			case 0 : $sChangefreq = 'always'; break;
			case 1 : $sChangefreq = 'hourly'; break;
			default:
			case 2 : $sChangefreq = 'daily'; break;
			case 3 : $sChangefreq = 'weekly'; break;
			case 4 : $sChangefreq = 'monthly'; break;
			case 5 : $sChangefreq = 'yearly'; break;
			case 6 : $sChangefreq = 'never'; break;
		}

		$this->_getOut()->write(
			"<url>\n" .
			"<loc>{$loc}</loc>\n" .
			"<changefreq>" . Core_Str::xml($sChangefreq) . "</changefreq>\n" .
			"<priority>" . Core_Str::xml($priority) . "</priority>\n" .
			"</url>\n"
		);

		$this->_inFile++;

		return $this;
	}

	public function execute()
	{
		$this->_close();

		if ($this->_createIndex)
		{
			echo '<?xml version="1.0" encoding="UTF-8"?>' . "\n";

			$oSite_Alias = $this->_oSite->getCurrentAlias();

			echo '<sitemapindex xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">'."\n";
			foreach ($this->_aIndexedFiles as $filename)
			{
				echo "<sitemap>\n";
				echo "<loc>http://{$oSite_Alias->name}/{$filename}</loc>";
				echo "<lastmod>" . date('Y-m-d') . "</lastmod>";
				echo "</sitemap>\n";
			}

			echo '</sitemapindex>';
		}

		return $this;
	}
}