<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Abstract skin
 *
 * @package HostCMS 6\Core
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
abstract class Core_Skin
{
	abstract public function header();

	abstract public function footer();

	abstract public function index();

	protected $_skinName = 'default';
	public function skinName($skinName)
	{
		$this->_skinName = $skinName;
		return $this;
	}
	public function getSkinName()
	{
		return $this->_skinName;
	}

	protected $_mode = NULL;
	public function setMode($mode)
	{
		$this->_mode = $mode;
		return $this;
	}

	public function getMode()
	{
		return $this->_mode;
	}

	protected $_title;
	public function title($title)
	{
		$this->_title = $title;
		return $this;
	}

	protected $_js = array();
	public function addJs($path)
	{
		$this->_js[] = $path;
		return $this;
	}

	public function getJs()
	{
		return $this->_js;
	}

	protected $_css = array();
	public function addCss($path)
	{
		$this->_css[] = $path;
		return $this;
	}

	public function getCss()
	{
		return $this->_css;
	}

	protected $_answer = NULL;

	/**
	 * Constructor.
	 */
	public function __construct()
	{
		$sAnswerName = 'Skin_' . ucfirst($this->_skinName) . '_Answer';
		$this->_answer = new $sAnswerName();
	}

	/**
	 * The singleton instances.
	 * @var array
	 */
	static public $instance = array();

	static public function instance($name = NULL)
	{
		is_null($name) && $name = Core::$mainConfig['skin'];

		if (!is_string($name))
		{
			throw new Core_Exception('Wrong argument type (expected String)');
		}

		if (!isset(self::$instance[$name]))
		{
			$skin = 'Skin_' . ucfirst($name);
			self::$instance[$name] = new $skin();
		}

		return self::$instance[$name];
	}

	public function answer()
	{
		return $this->_answer;
	}

	protected function _getTimestamp()
	{
		$currentVersion = defined('CURRENT_VERSION') ? CURRENT_VERSION : '6.0';
		return abs(Core::crc32($currentVersion . $currentVersion));
	}

	public function getSkinModuleName($modulePath)
	{
		return "Skin_{$this->_skinName}_Module_{$modulePath}_Module";
	}

	public function _getAllowedModules()
	{
		$oUser = Core_Entity::factory('User')->getCurrent();

		if (!$oUser)
		{
			return array();
		}

		$oModule = Core_Entity::factory('Module');
		$oModule->queryBuilder()->where('active', '=', 1);

		if ($oUser->superuser == 0)
		{
			$oModule->queryBuilder()
				->select('modules.*')
				->join('user_modules', 'modules.id', '=', 'user_modules.module_id',
				array(
					array('AND' => array('user_group_id', '=', $oUser->user_group_id)),
					array('AND' => array('site_id', '=', CURRENT_SITE))
				));
		}

		return $oModule->findAll();
	}

	protected $_lng = NULL;
	public function getLng()
	{
		if (is_null($this->_lng))
		{
			if (Core::isInit())
			{
				$this->_lng = htmlspecialchars(
					Core_Entity::factory('Admin_Language')->getCurrent()->shortname
				);
			}
			else
			{
				$this->_lng = Core_I18n::instance()->getLng();
			}
		}
		return $this->_lng;
	}

	public function setLng($lng)
	{
		$this->_lng = $lng;
		return $this;
	}
}