<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Modules.
 *
 * @package HostCMS 6\Module
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
class Module_Model extends Core_Entity
{
	public $version = NULL;
	public $date = NULL;

	public $Core_Module = NULL;

	protected $_sorting = array(
		'modules.sorting' => 'ASC'
	);

	/**
	 * Constructor.
	 */
	public function __construct($id = NULL)
	{
		parent::__construct($id);

		if (is_null($id))
		{
			$oUserCurrent = Core_Entity::factory('User', 0)->getCurrent();
			$this->_preloadValues['user_id'] = is_null($oUserCurrent) ? 0 : $oUserCurrent->id;
		}

		$this->_getModuleInformation();
	}

	protected function _getModuleInformation()
	{
		if (is_null($this->version) && !is_null($this->path))
		{
			$this->Core_Module = Core_Module::factory($this->path);
			if ($this->Core_Module)
			{
				$this->version = $this->Core_Module->version;
				$this->date = $this->Core_Module->date;
			}
		}
	}

	public function find($primaryKey = NULL, $bCache = TRUE)
	{
		$return = parent::find($primaryKey, $bCache);
		$this->_getModuleInformation();
		return $return;
	}

	/**
	 * List of preloaded values
	 * @var array
	 */
	protected $_preloadValues = array(
		'sorting' => 0
	);

	public function getConfigFilePath()
	{
		$classPath = Core::getClassPath($this->path);
		return Core::$modulesPath . dirname($classPath) . DIRECTORY_SEPARATOR . 'config' . DIRECTORY_SEPARATOR . 'config.php';
	}

	public function saveConfigFile($content)
	{
		$this->save();

		$configFilePath = $this->getConfigFilePath();
		$dir = dirname($configFilePath);
		if (!is_dir($dir))
		{
			Core_File::mkdir($dir, CHMOD, $recursive = TRUE);
		}

		$content = trim($content);
		Core_File::write($configFilePath, $content);
	}

	public function loadConfigFile()
	{
		$path = $this->getConfigFilePath();

		if (is_file($path))
		{
			return Core_File::read($path);
		}
		else
		{
			return NULL;
		}
	}

	public function getByPath($path)
	{
		$this->queryBuilder()
			->clear()
			->where('path', '=', $path)
			->limit(1);

		$aModules = $this->findAll();

		if (isset($aModules[0]))
		{
			return $aModules[0];
		}

		return NULL;
	}

	/**
	 * Change module status
	 * @return Module_Model
	 */
	public function changeActive()
	{
		$this->active = 1 - $this->active;
		$this->save();

		return $this->setupModule();
	}

	public function setupModule()
	{
		$this->active && $this->install();
		!$this->active && $this->uninstall();
		return $this;
	}

	public function install()
	{
		$path = $this->path . '_Module';
		if (class_exists($path))
		{
			$objectModule = new $path();
			method_exists($objectModule, 'install') && $objectModule->install();
		}

		return $this;
	}

	public function uninstall()
	{
		$path = $this->path . '_Module';
		if (class_exists($path))
		{
			$objectModule = new $path();
			method_exists($objectModule, 'uninstall') && $objectModule->uninstall();
		}

		return $this;
	}
}