<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Properties.
 *
 * @package HostCMS 6\Property
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
class Property_Value_File_Model extends Core_Entity
{
	/**
	 * Model name
	 * @var mixed
	 */
	protected $_modelName = 'property_value_file';

	/**
	 * Disable markDeleted()
	 * @var mixed
	 */
	protected $_marksDeleted = NULL;

	/**
	 * Belongs to relations
	 * @var array
	 */
	protected $_belongsTo = array(
		'property' => array()
	);

	/**
	 * List of preloaded values
	 * @var array
	 */
	protected $_preloadValues = array(
		'file' => '',
		'file_name' => '',
		'file_description' => '',
		'file_small' => '',
		'file_small_name' => '',
		'file_small_description' => ''
	);

	protected $_dir = NULL;
	public function setDir($dir)
	{
		$this->_dir = $dir;
		return $this;
	}

	protected $_href = NULL;
	public function setHref($href)
	{
		$this->_href = $href;
		return $this;
	}

	/*public function setValue($value)
	{
		$this->value = intval($value);
		return $this;
	}*/

	/**
	 * Delete object from database
	 * @param mixed $primaryKey primary key for deleting object
	 * @return Property_Value_File_Model
	 */
	public function delete($primaryKey = NULL)
	{
		if (is_null($primaryKey))
		{
			$primaryKey = $this->getPrimaryKey();
		}

		$this->id = $primaryKey;

		if ($this->_dir)
		{
			$this
				->deleteLargeFile()
				->deleteSmallFile();
		}

		return parent::delete($primaryKey);
	}

	public function getLargeFilePath()
	{
		return $this->_dir . Core_File::convertFileNameToLocalEncoding($this->file);
	}

	public function getLargeFileHref()
	{
		return $this->_href . Core_File::convertFileNameToLocalEncoding($this->file);
	}

	public function deleteLargeFile()
	{
		$path = $this->getLargeFilePath();
		try
		{
			Core_File::delete($path);
		} catch (Exception $e) {};
		$this->file = '';
		$this->file_name = '';
		//$this->file_description = '';
		return $this->save();
	}

	public function getSmallFilePath()
	{
		return $this->_dir . Core_File::convertFileNameToLocalEncoding($this->file_small);
	}

	public function getSmallFileHref()
	{
		return $this->_href . Core_File::convertFileNameToLocalEncoding($this->file_small);
	}

	public function deleteSmallFile()
	{
		$path = $this->getSmallFilePath();

		try
		{
			Core_File::delete($path);
		} catch (Exception $e) {};
		$this->file_small = '';
		$this->file_small_name = '';
		//$this->file_small_description = '';
		return $this->save();
	}

	protected $_tagName = 'property_value';

	public function getXml()
	{
		if (!is_null($this->_href))
		{
			$this->clearXmlTags();

			if ($this->file != '')
			{
				$this->addXmlTag('file_path', $this->_href . rawurlencode($this->file));
			}

			if ($this->file_small != '')
			{
				$this->addXmlTag('file_small_path', $this->_href . rawurlencode($this->file_small));
			}
		}

		$this->clearXmlTags()
			->addXmlTag('property_dir_id', $this->Property->property_dir_id)
			->addXmlTag('tag_name', $this->Property->tag_name);

		return parent::getXml();
	}
}