<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Экспорт в YandexMarket для магазина.
 *
 * Доступные методы:
 * group($id) идентификатор группы магазина, если FALSE, то вывод товаров осуществляется из всех групп
 * groupsProperties(TRUE) выводить значения дополнительных свойств групп, по умолчанию NULL
 * itemsProperties(TRUE) выводить значения дополнительных свойств товаров, по умолчанию NULL
 * tags(TRUE) выводить метки
 * offset($offset) смещение, с которого выводить товары. По умолчанию 0
 * limit($limit) количество выводимых товаров
 *
 * <code>
 * $Shop_Controller_Show = new Shop_Controller_Show(
 * 	Core_Entity::factory('Shop', 1)
 * );
 *
 * $Shop_Controller_Show
 * 	->xsl(
 * 		Core_Entity::factory('Xsl')->getByName('СписокНовостейНаГлавной')
 * 	)
 * 	->limit(5)
 * 	->show();
 * </code>
 *
 * @package HostCMS 6\Shop
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
class Shop_Controller_YandexMarket extends Core_Controller
{
	/**
	 * Allowed object properties
	 * @var array
	 */
	protected $_allowedProperties = array()	;

	// Shop's items object
	protected $_Shop_Items = NULL;

	// Shop's group object
	protected $_Shop_Groups = NULL;

	protected $_aSiteuserGroups = array();

	/**
	 * Constructor.
	 */
	public function __construct(Shop_Model $oShop)
	{
		parent::__construct($oShop->clearEntities());

		$this->_Shop_Items = $oShop->Shop_Items;

		$siteuser_id = 0;

		$this->_aSiteuserGroups = array(0, -1);
		if (Core::moduleIsActive('siteuser'))
		{
			$oSiteuser = Core_Entity::factory('Siteuser')->getCurrent();

			if ($oSiteuser)
			{
				$siteuser_id = $oSiteuser->id;

				$aSiteuser_Groups = $oSiteuser->Siteuser_Groups->findAll();
				foreach ($aSiteuser_Groups as $oSiteuser_Group)
				{
					$this->_aSiteuserGroups[] = $oSiteuser_Group->id;
				}
			}
		}

		switch ($oShop->items_sorting_direction)
		{
			case 1:
				$items_sorting_direction = 'DESC';
			break;
			case 0:
			default:
				$items_sorting_direction = 'ASC';
		}

		// Определяем поле сортировки информационных элементов
		switch ($oShop->items_sorting_field)
		{
			case 1:
				$this->_Shop_Items
					->queryBuilder()
					->orderBy('shop_items.name', $items_sorting_direction);
				break;
			case 2:
				$this->_Shop_Items
					->queryBuilder()
					->orderBy('shop_items.sorting', $items_sorting_direction)
					->orderBy('shop_items.name', $items_sorting_direction);
				break;
			case 0:
			default:
				$this->_Shop_Items
					->queryBuilder()
					->orderBy('shop_items.datetime', $items_sorting_direction);
		}

		$dateTime = Core_Date::timestamp2sql(time());
		$this->_Shop_Items
			->queryBuilder()
			->select('shop_items.*')
			->join('shop_groups', 'shop_groups.id', '=', 'shop_items.shop_group_id',
				array(
						array('AND' => array('shop_groups.active', '=', 1)),
						array('OR' => array('shop_items.shop_group_id', '=', 0))
					)
			)
			->where('shop_items.shortcut_id', '=', 0)
			->where('shop_items.active', '=', 1)
			->where('shop_items.siteuser_id', 'IN', $this->_aSiteuserGroups)
			->open()
			->where('shop_items.start_datetime', '<', $dateTime)
			->setOr()
			->where('shop_items.start_datetime', '=', '0000-00-00 00:00:00')
			->close()
			->setAnd()
			->open()
			->where('shop_items.end_datetime', '>', $dateTime)
			->setOr()
			->where('shop_items.end_datetime', '=', '0000-00-00 00:00:00')
			->close()
			->where('shop_items.yandex_market', '=', 1)
			->where('shop_items.price', '>', 0);

		$this->_Shop_Groups = $oShop->Shop_Groups;
		$this->_Shop_Groups
			->queryBuilder()
			->where('shop_groups.siteuser_group_id', 'IN', $this->_aSiteuserGroups)
			->where('active', '=', 1);

		switch ($oShop->groups_sorting_direction)
		{
			case 0:
				$groups_sorting_direction = 'ASC';
				break;
			case 1:
			default:
				$groups_sorting_direction = 'DESC';
		}

		// Определяем поле сортировки групп
		switch ($oShop->groups_sorting_field)
		{
			case 0:
				$this->_Shop_Groups
					->queryBuilder()
					->orderBy('shop_groups.name', $groups_sorting_direction);
				break;
			case 1:
			default:
				$this->_Shop_Groups
					->queryBuilder()
					->orderBy('shop_groups.sorting', $groups_sorting_direction);
				break;
		}
	}

	/**
	 * Get items set
	 * @return Shop_Item_Model
	 */
	public function shopItems()
	{
		return $this->_Shop_Items;
	}

	/**
	 * Get groups set
	 * @return Shop_Item_Model
	 */
	public function shopGroups()
	{
		return $this->_Shop_Groups;
	}

	/**
	 * Show built data
	 * @return self
	 */
	public function show()
	{
		$oShop = $this->getEntity();
		$oSite = $oShop->Site;

		Core_Page::instance()->response
			->header('Content-Type', "text/xml; charset={$oSite->coding}")
			->sendHeaders();

		echo '<?xml version="1.0" encoding="' . $oSite->coding . '"?>' . "\n";
		echo '<!DOCTYPE yml_catalog SYSTEM "shops.dtd">' . "\n";
		echo '<yml_catalog date="' . date("Y-m-d H:i") . '">' . "\n";
		echo "<shop>\n";

		// Название магазина
		$shop_name = trim(
			!empty($oSite->yandex_market_name)
				? $oSite->yandex_market_name
				: $oSite->name
		);

		echo "<name>" . Core_Str::xml(mb_substr($shop_name, 0, 20)) . "</name>\n";

		// Название компании.
		echo "<company>" . Core_Str::xml($oShop->Shop_Company->name) . "</company>\n";

		$oSite_Alias = $oSite->getCurrentAlias();
		$sShopPath = 'http://' . $oSite_Alias->name . $oShop->Structure->getPath();

		echo "<url>" . Core_Str::xml($sShopPath) . "</url>\n";
		echo "<platform>HostCMS</platform>\n";
		echo "<version>" . Core_Str::xml(CURRENT_VERSION) . "</version>\n";

		/* Валюты */
		echo '<currencies>'."\n";
		$aShop_Currencies = Core_Entity::factory('Shop_Currency')->findAll();

		$aCurrenciesCodes = array(
			'RUR',
			'RUB',
			'USD',
			'BYR',
			'KZT',
			'EUR',
			'UAH',
		);

		foreach ($aShop_Currencies as $oShop_Currency)
		{
			if (trim($oShop_Currency->code) != ''
			&& in_array($oShop_Currency->code, $aCurrenciesCodes))
			{
					echo  '<currency id="' . $oShop_Currency->code .
							'" rate="' . $oShop_Currency->exchange_rate .'"'."/>\n";
			}
		}
		echo '</currencies>'."\n";

		/* Категории */
		$aShop_Groups = $this->_Shop_Groups->findAll();
		if (count($aShop_Groups))
		{
			echo "<categories>\n";
			foreach ($aShop_Groups as $oShop_Group)
			{
				$group_parent_id = $oShop_Group->parent_id == '' || $oShop_Group->parent_id == 0 ? '' : ' parentId="' . $oShop_Group->parent_id . '"';

				echo  '<category id="' . $oShop_Group->id . '"' . $group_parent_id . '>' . $oShop_Group->name ."</category>\n";
			}
			echo "</categories>\n";
		}

		flush();

		/* Товары */
		echo "<offers>\n";

		$aShop_Items = $this->_Shop_Items->findAll();

		foreach ($aShop_Items as $oShop_Item)
		{
			/* Устанавливаем атрибуты тега <offer>*/
			$tag_bid = $oShop_Item->yandex_market_bid
				? ' bid="' . $oShop_Item->yandex_market_bid . '"'
				: '';

			$tag_cid = $oShop_Item->yandex_market_cid
				? ' cbid="' . $oShop_Item->yandex_market_cid . '"'
				: '';

			$oShop_Warehouse_Item = $oShop_Item->Shop_Warehouse_Items->getByShopItemId($oShop_Item->id);
			$available = !is_null($oShop_Warehouse_Item) && $oShop_Warehouse_Item->count > 0 ? 'true' : 'false';

			echo  '<offer id="' . $oShop_Item->id . '"'. $tag_bid . $tag_cid . " available=\"{$available}\">\n";

			/* URL */
			echo '<url>' . $sShopPath . $oShop_Item->getPath() . '</url>'."\n";

			// Определяем цену со скидкой.
			$price = array();
			$aShop_Item_Discounts = $oShop_Item->Shop_Item_Discounts->findAll();
			if (count($aShop_Item_Discounts))
			{
				// определяем количество скидок на товар
				$percent = 0;

				// Цикл по идентификаторам скидок для товара
				foreach ($aShop_Item_Discounts as $oShop_Item_Discount)
				{
					if ($oShop_Item_Discount->Shop_Discount->isActive())
					{
						$price['discounts'][] = $oShop_Item_Discount->Shop_Discount;
						$percent += $oShop_Item_Discount->Shop_Discount->percent;
					}
				}

				// определяем суммарную величину скидки в валюте
				$price['discount'] = $oShop_Item->price * $percent / 100;

				// вычисляем цену со скидкой как ее разность с величиной скидки
				$price['price_discount'] = $oShop_Item->price - $price['discount'];
			}
			else
			{
				// если скидок нет, то price_discount положим равным price
				$price['price_discount'] = $oShop_Item->price;
				$price['discount'] = 0;
			}

			/* Цена */
			echo  '<price>' . $price['price_discount'] . '</price>'. "\n";

			/* CURRENCY */
			if ($oShop_Item->Shop_Currency->code != '')
			{
				echo  '<currencyId>'. $oShop_Item->Shop_Currency->code . '</currencyId>'. "\n";
			}

			/* Идентификатор категории */
			// Основной товар
			if ($oShop_Item->modification_id == 0)
			{
				$categoryId = $oShop_Item->shop_group_id;
			}
			else // Модификация, берем ID родительской группы
			{
				$categoryId = $oShop_Item->Modification->Shop_Group->id
					? $oShop_Item->Modification->Shop_Group->id
					: 0;
			}
			echo  '<categoryId>' . $categoryId . '</categoryId>'."\n";

			/* Указывает код товара производителя */
			$bVendorCode = $oShop_Item->vendorcode != '';

			/* PICTURE */
			// При указании VendorCode картинки нельзя
			if (!$bVendorCode && $oShop_Item->image_large != "")
			{
					echo  '<picture>' . 'http://' . $oSite_Alias->name . $oShop_Item->getLargeFileHref() . '</picture>'. "\n";
			}

			if (mb_strlen($oShop_Item->name) > 0)
			{
				/* NAME */
				echo  '<name>' . $oShop_Item->name . '</name>'. "\n";

				/* vendorCode */
				if ($bVendorCode)
				{
					echo  '<vendorCode>' . $oShop_Item->vendorcode . '</vendorCode>'."\n";
				}
			}

			/* DESCRIPTION */
			if (!empty($oShop_Item->description))
			{
				echo  '<description>' . html_entity_decode(strip_tags($oShop_Item->description), ENT_COMPAT, 'UTF-8') . '</description>'."\n";
			}

			/* sales_notes */
			$sales_notes = mb_strlen($oShop_Item->yandex_market_sales_notes) > 0
				? $oShop_Item->yandex_market_sales_notes
				: $oShop->yandex_market_sales_notes_default;

			echo  '<sales_notes>' . html_entity_decode(strip_tags($sales_notes), ENT_COMPAT, 'UTF-8') . '</sales_notes>'."\n";
			echo '</offer>'."\n";
		}
		echo '</offers>'."\n";
		echo "</shop>\n";
		echo '</yml_catalog>' . "\n";
	}
}