<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Online shop.
 *
 * @package HostCMS 6\Shop
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
class Shop_Delivery_Condition_Model extends Core_Entity
{
	public $currency_name = '';
	public $orderfield = '';

	protected $_belongsTo = array(
		'shop_delivery' => array(),
		'shop_country' => array(),
		'shop_country_location' => array(),
		'shop_country_location_city' => array(),
		'shop_country_location_city_area' => array(),
		'shop_tax' => array(),
		'shop_currency' => array()
	);

	/**
	 * List of preloaded values
	 * @var array
	 */
	protected $_preloadValues = array(
		'max_weight' => 0.00,
		'min_weight' => 0.00,
		'max_price' => 0.00,
		'min_price' => 0.00,
		'price' => 0.00
	);

	/**
	 * Forbidden tags. If list of tags is empty, all tags will show.
	 * @var array
	 */
	protected $_forbiddenTags = array(
		'price',
	);

	/**
	 * Constructor.
	 */
	public function __construct($id = NULL)
	{
		parent::__construct($id);

		if (is_null($id))
		{
			$oUserCurrent = Core_Entity::factory('User', 0)->getCurrent();
			$this->_preloadValues['user_id'] = is_null($oUserCurrent) ? 0 : $oUserCurrent->id;
		}
	}

	public function getPriceArray()
	{
		$oShop = $this->Shop_Delivery->Shop;

		// Определяем коэффициент пересчета
		$fCurrencyCoefficient = $this->shop_currency_id > 0 && $oShop->shop_currency_id > 0
			? Shop_Controller::instance()->getCurrencyCoefficientInShopCurrency(
				$this->Shop_Currency, $oShop->Shop_Currency
			)
			: 0;

		$price = $this->price * $fCurrencyCoefficient;

		$tax = 0;
		$rate = 0;
		if ($this->shop_tax_id)
		{
			$oShop_Tax = $this->Shop_Tax;

			if ($oShop_Tax->id)
			{
				$rate = $oShop_Tax->rate;

				// Если он не входит в цену
				if ($oShop_Tax->tax_is_included == 0)
				{
					// То считаем цену с налогом
					$tax = $oShop_Tax->rate / 100 * $price;
				}
				else
				{
					$tax = $price / (100 + $oShop_Tax->rate) * $oShop_Tax->rate;
					$price -= $tax;
				}
			}
		}

		$oShop_Controller = Shop_Controller::instance();

		return array(
			'price' => $oShop_Controller->round($price),
			'tax' => $oShop_Controller->round($tax),
			'rate' => $rate
		);
	}

	/**
	 * Get XML for entity and children entities
	 * @return string
	 */
	public function getXml()
	{
		$aPrice = $this->getPriceArray();

		$this->clearXmlTags()
			->addXmlTag('price', $aPrice['price'] + $aPrice['tax']);

		return parent::getXml();
	}
}