<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Online shop.
 *
 * @package HostCMS 6\Shop
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
class Shop_Item_Import_Cml_Controller extends Core_Servant_Properties
{
	private $_aReturn = array(
		'insertDirCount' => 0,
		'insertItemCount' => 0,
		'updateDirCount' => 0,
		'updateItemCount' => 0
	);

	private $_oSimpleXMLElement = NULL;
	private $aPredefinedBaseProperties = array(
		"HOSTCMS_TITLE",
		"HOSTCMS_DESCRIPTION",
		"HOSTCMS_KEYWORDS",
		"HOSTCMS_МЕТКИ",
		"YANDEX_MARKET",
		"ПРОДАВЕЦ",
		"ПРОИЗВОДИТЕЛЬ",
		"АКТИВНОСТЬ");
	private $aBaseProperties = array();

	protected $_allowedProperties = array(
		'iShopId',
		'iShopGroupId',
		'sShopDefaultPriceName',
		'sShopDefaultPriceGUID',
		'sPicturesPath',
		'importAction'
	);

	protected $_aPropertyValues = array();

	/**
	 * Constructor.
	 */
	public function __construct($sXMLFilePath)
	{
		parent::__construct();

		$this->_oSimpleXMLElement = new SimpleXMLElement(Core_File::read($sXMLFilePath));
		$this->sShopDefaultPriceName = 'РОЗНИЧНАЯ';
		$this->sShopDefaultPriceGUID = '';
	}

	private function _importGroups($oXMLNode, $iParentId = 0)
	{
		foreach($oXMLNode->xpath('Группа') as $oXMLGroupNode)
		{
			$oShopGroup = Core_Entity::factory('Shop', $this->iShopId)->Shop_Groups->getByGuid(strval($oXMLGroupNode->Ид));
			is_null($oShopGroup) && $oShopGroup = Core_Entity::factory('Shop_Group');
			$oShopGroup->name = strval($oXMLGroupNode->Наименование);
			$oShopGroup->guid = strval($oXMLGroupNode->Ид);
			$oShopGroup->parent_id = $iParentId;
			$oShopGroup->shop_id = $this->iShopId;

			if(is_null($oShopGroup->id))
			{
				$this->_aReturn['insertDirCount']++;
			}
			else
			{
				$this->_aReturn['updateDirCount']++;
			}

			$oShopGroup->save();

			$oSubGroups = $oXMLGroupNode->Группы;
			is_object($oSubGroups) && $this->_importGroups($oSubGroups, $oShopGroup->id);
		}
		return $this;
	}

	private function _addTax($oTax)
	{
		$sTaxName = strval($oTax->Наименование);
		$sTaxRate = strval($oTax->Ставка);
		$sTaxGUID = md5(mb_strtoupper($sTaxName));

		$oShopTax = Core_Entity::factory('Shop_Tax')->getByGuid($sTaxGUID);

		if(is_null($oShopTax))
		{
			$oShopTax = Core_Entity::factory('Shop_Tax');
			$oShopTax->name = $sTaxName;
			$oShopTax->guid = $sTaxGUID;
		}

		$oShopTax->rate = ($sTaxRate == '' ? 0 : $sTaxRate);

		$oShopTax->tax_is_included = 0;

		$oShopTax->save();

		return $oShopTax;
	}

	private function _addItemProperty($aPropertyData)
	{
		$sPropertyGUID = Core_Array::get($aPropertyData, 'guid', FALSE);
		$sPropertyName = Core_Array::get($aPropertyData, 'name', FALSE);
		$iGroupForAccepting = Core_Array::get($aPropertyData, 'group', -1);

		$oShop = Core_Entity::factory('Shop', $this->iShopId);

		if($sPropertyGUID === FALSE && $sPropertyName === FALSE) throw new Core_Exception(Core::_('Shop_Item.error_property_guid'));

		$sPropertyGUID === FALSE && $sPropertyGUID = md5(mb_strtoupper($sPropertyName));

		$oShop_Item_Property_List = Core_Entity::factory('Shop_Item_Property_List', $this->iShopId);
		$oProperty = $oShop_Item_Property_List->Properties->getByGuid($sPropertyGUID);

		is_null($oProperty) && $oShop_Item_Property_List->add($oProperty = Core_Entity::factory('Property')->name($sPropertyName)->type(1)->guid($sPropertyGUID));

		$oProperty
			->name($sPropertyName)
			->tag_name(Core_Str::transliteration($sPropertyName))
			->save();

		if($iGroupForAccepting == -1)
		{
			$aShopGroups = $oShop->Shop_Groups->findAll();

			$aShopGroups[] = Core_Entity::factory('Shop_Group', 0);

			foreach($aShopGroups as $oShopGroup)
			{
				$oShop->Shop_Item_Property_For_Groups->allowAccess($oProperty->Shop_Item_Property->id, $oShopGroup->id);
			}
		}
		else
		{
			$oShop->Shop_Item_Property_For_Groups->allowAccess($oProperty->Shop_Item_Property->id, $iGroupForAccepting);
		}

		return $oProperty;
	}

	public function import()
	{
		Core_Event::notify('Shop_Item_Import_Cml_Controller.onBeforeImport', $this);

		if(is_null($this->iShopId))
		{
			throw new Core_Exception(Core::_('Shop_Item.error_shop_id'));
		}

		if(is_null($this->iShopGroupId))
		{
			throw new Core_Exception(Core::_('Shop_Item.error_parent_directory'));
		}

		if($this->importAction == 0)
		{
			Core_QueryBuilder::update('shop_groups')
				->set('deleted', 1)
				->where('shop_id', '=', $this->_oCurrentShop->id)
				->execute();

			Core_QueryBuilder::update('shop_items')
				->set('deleted', 1)
				->where('shop_id', '=', $this->_oCurrentShop->id)
				->execute();
		}

		$oShop = Core_Entity::factory('Shop', $this->iShopId);

		// Файл import.xml
		if(count((array)$this->_oSimpleXMLElement->Классификатор))
		{
			$classifier = $this->_oSimpleXMLElement->Классификатор;

			// Импортируем группы товаров
			is_object($classifier->Группы) && $this->_importGroups($classifier->Группы, $this->iShopGroupId);

			// Импортируем дополнительные свойства товаров
			foreach($classifier->xpath('Свойства/Свойство') as $oItemProperty)
			{
				$sPropertyName = strval($oItemProperty->Наименование);

				foreach($oItemProperty->xpath('ВариантыЗначений/Справочник') as $oValue)
				{
					$this->_aPropertyValues[strval($oValue->ИдЗначения)] = strval($oValue->Значение);
				}

				if(in_array(mb_strtoupper($sPropertyName), $this->aPredefinedBaseProperties))
				{
					// Основное свойство товара
					$this->aBaseProperties[strval($oItemProperty->Ид)] = strval($oItemProperty->Наименование);
				}
				else
				{
					// Дополнительное свойство товара
					$this->_addItemProperty(array(
						//'guid' => md5(mb_strtoupper(strval($oItemProperty->Ид))), ???
						'guid' => strval($oItemProperty->Ид),
						'name' => strval($oItemProperty->Наименование),
						'group' => -1
					));
				}
			}

			foreach($this->_oSimpleXMLElement->Каталог->xpath('Товары/Товар') as $oItem)
			{
				$sGUID = strval($oItem->Ид);
				$sGUIDmod = FALSE;

				if(strpos($sGUID, '#') !== FALSE)
				{
					$sTmp = explode('#', $sGUID);
					$sGUID = $sTmp[0];
					$sGUIDmod = $sTmp[1];
				}

				$oShopItem = $oShop->Shop_Items->getByGuid($sGUID);

				$sItemName = strval($oItem->Наименование);

				if(is_null($oShopItem))
				{
					// Создаем товар
					$oShopItem = Core_Entity::factory('Shop_Item')->guid($sGUID);

					// Если товар - модификация, для основного товара оставляем лишь базовые данные, название и идентификатор магазина/группы товаров
					if($sGUIDmod)
					{
						$oShopItem->name = $sItemName;
						$oShopItem->shop_id($this->iShopId)->save();

						$iShopParentItemId = $oShopItem->id;
						$oShopItem = Core_Entity::factory('Shop_Item')
							->guid($sGUIDmod)
							->modification_id($iShopParentItemId)
							->shop_group_id(0);
					}

					$this->_aReturn['insertItemCount']++;
				}
				else
				{
					$this->_aReturn['updateItemCount']++;
				}

				$oShopItem->marking = strval($oItem->Артикул);
				$oShopItem->name = $sItemName;

				if(is_array($aTmp = $oItem->xpath('Группы'))
				&& count($aTmp) > 0
				&& !is_null($oShop_Group = $oShop->Shop_Groups->getByGuid(strval($aTmp[0]->Ид))))
				{
					// Группа указана в файле и существует в магазине
					$sGUIDmod === FALSE
						? $oShopItem->shop_group_id = $oShop_Group->id
							: $oShopItem->Modification->shop_group_id($oShop_Group->id)->save();
				}
				else
				{
					// Группа не указана в файле, кладем в корень (iShopGroupId)
					$sGUIDmod === FALSE
						? $oShopItem->shop_group_id = $this->iShopGroupId
						: $oShopItem->Modification->shop_group_id($this->iShopGroupId)->save();
				}

				$oShopItem->shop_id = $this->iShopId;

				if(($oShopItem->id
				&& $this->importAction == 1
				&& !is_null($oShopItem->name)))
				{
					$oShopItem->save();
				}
				elseif(!is_null($oShopItem->name))
				{
					$oShopItem->save();
				}
				else
				{
					throw new Core_Exception(Core::_('Shop_Item.error_save_without_name'));
				}

				
				// Обрабатываем описание товара
				foreach($oItem->xpath('Описание') as $DescriptionData)
				{
					$oShopItem->description = strval($DescriptionData);
					$oShopItem->save();
				}

				$bFirstPicture = TRUE;

				// Обрабатываем изображения для товара
				foreach($oItem->xpath('Картинка') as $PictureData)
				{
					$PictureData = strval($PictureData);
					if(Core_File::isValidExtension(
					$PictureData,
					Core::$mainConfig['availableExtension']))
					{
						// Папка назначения
						$sDestinationFolder = $oShopItem->getItemPath();

						if(!$bFirstPicture)
						{
							$sTmp = explode('_', basename($PictureData));
							$sTmp[1] = basename($sTmp[1], '.'.Core_File::getExtension($sTmp[1]));

							$oShop_Item_Property_List = Core_Entity::factory('Shop_Item_Property_List', $oShop->id);
							$oProperty = $oShop_Item_Property_List->Properties->getByGuid($sTmp[1]);

							if(is_null($oProperty))
							{
								$oProperty = Core_Entity::factory('Property');
								$oProperty->name = $sTmp[1];
								$oProperty->type = 2;
								$oProperty->tag_name = $sTmp[1];
								$oProperty->guid = $sTmp[1];

								// Для вновь создаваемого допсвойства размеры берем из магазина
								$oProperty->image_large_max_width = $oShop->image_large_max_width;
								$oProperty->image_large_max_height = $oShop->image_large_max_height;
								$oProperty->image_small_max_width = $oShop->image_small_max_width;
								$oProperty->image_small_max_height = $oShop->image_small_max_height;

								$oShop_Item_Property_List->add($oProperty);
							}

							if(is_null(Core_Entity::factory('Shop', $oShop->id)
								->Shop_Item_Property_For_Groups
								->getByShopItemPropertyIdAndGroupId($oProperty->Shop_Item_Property->id, $oShopItem->shop_group_id)))
							{
								// Свойство не доступно текущей группе, делаем его доступным
								Core_Entity::factory('Shop_Item_Property_For_Group')
									->shop_group_id($oShopItem->shop_group_id)
									->shop_item_property_id($oProperty->Shop_Item_Property->id)
									->shop_id($oShop->id)
									->save();
							}

							$aPropertyValues = $oProperty->getValues($oShopItem->id, FALSE);

							$oProperty_Value = isset($aPropertyValues[0])
								? $aPropertyValues[0]
								: $oProperty->createNewValue($oShopItem->id);

							if($oProperty_Value->file != '')
							{
								try
								{
									Core_File::delete($sDestinationFolder . $oProperty_Value->file);
								} catch (Exception $e) {}
							}

							// Удаляем старое малое изображение
							if($oProperty_Value->file_small != '')
							{
								try
								{
									Core_File::delete($sDestinationFolder . $oProperty_Value->file_small);
								} catch (Exception $e) {}
							}
						}
						else
						{
							if($oShopItem->image_large != '')
							{
								try
								{
									Core_File::delete($sDestinationFolder . $oShopItem->image_large);
								} catch (Exception $e) {}
							}

							// Удаляем старое малое изображение
							if($oShopItem->image_small != '')
							{
								try
								{
									Core_File::delete($sDestinationFolder . $oShopItem->image_small);
								} catch (Exception $e) {}
							}
						}

						// Удаляем папку назначения вместе со всеми старыми файлами
						//Core_File::deleteDir($sDestinationFolder);

						// Создаем папку назначения
						$oShopItem->createDir();

						if($this->sPicturesPath == '')
						{
							$this->sPicturesPath = CMS_FOLDER;
						}

						// Файл-источник
						$sSourceFile = $this->sPicturesPath . $PictureData;
						$sSourceFileBaseName = basename($PictureData);

						if(!$oShop->change_filename)
						{
							$sTargetFileName = $sSourceFileBaseName;
						}
						else
						{
							$sTargetFileExtension = Core_File::getExtension($PictureData);

							if($sTargetFileExtension != '')
							{
								$sTargetFileExtension = ".{$sTargetFileExtension}";
							}

							if(!$bFirstPicture)
							{
								$sTargetFileName = "shop_property_file_{$oShopItem->id}_{$oProperty->id}{$sTargetFileExtension}";
							}
							else
							{
								$sTargetFileName = "shop_items_catalog_image{$oShopItem->id}{$sTargetFileExtension}";
							}
						}

						$aPicturesParam = array();
						$aPicturesParam['large_image_isset'] = TRUE;
						$aPicturesParam['large_image_source'] = $sSourceFile;
						$aPicturesParam['large_image_name'] = $sSourceFileBaseName;
						$aPicturesParam['large_image_target'] = $sDestinationFolder . $sTargetFileName;
						$aPicturesParam['watermark_file_path'] = $oShop->getWatermarkFilePath();
						$aPicturesParam['watermark_position_x'] = $oShop->watermark_default_position_x;
						$aPicturesParam['watermark_position_y'] = $oShop->watermark_default_position_y;
						$aPicturesParam['large_image_preserve_aspect_ratio'] = $oShop->preserve_aspect_ratio;
						$aPicturesParam['small_image_source'] = $aPicturesParam['large_image_source'];
						$aPicturesParam['small_image_name'] = $aPicturesParam['large_image_name'];
						$aPicturesParam['small_image_target'] = $sDestinationFolder . "small_{$sTargetFileName}";
						$aPicturesParam['create_small_image_from_large'] = TRUE;

						if(!$bFirstPicture)
						{
							$aPicturesParam['large_image_max_width'] = $oProperty->image_large_max_width;
							$aPicturesParam['large_image_max_height'] = $oProperty->image_large_max_height;
							$aPicturesParam['small_image_max_width'] = $oProperty->image_small_max_width;
							$aPicturesParam['small_image_max_height'] = $oProperty->image_small_max_height;
						}
						else
						{
							$aPicturesParam['large_image_max_width'] = $oShop->image_large_max_width;
							$aPicturesParam['large_image_max_height'] = $oShop->image_large_max_height;
							$aPicturesParam['small_image_max_width'] = $oShop->image_small_max_width;
							$aPicturesParam['small_image_max_height'] = $oShop->image_small_max_height;
						}

						$aPicturesParam['small_image_watermark'] = $oShop->watermark_default_use_small_image;
						$aPicturesParam['small_image_preserve_aspect_ratio'] = $aPicturesParam['large_image_preserve_aspect_ratio'];

						$aPicturesParam['large_image_watermark'] = $oShop->watermark_default_use_large_image;

						try
						{
							$result = Core_File::adminUpload($aPicturesParam);
						}
						catch (Exception $exc)
						{
							$result = array('large_image' => FALSE, 'small_image' => FALSE);
						}

						if ($result['large_image'])
						{
							if(!$bFirstPicture)
							{
								$oProperty_Value->file = $sTargetFileName;
								$oProperty_Value->file_name = '';
								$oProperty_Value->save();
							}
							else
							{
								$oShopItem->image_large = $sTargetFileName;
								$oShopItem->setLargeImageSizes();
							}
						}

						if ($result['small_image'])
						{
							if(!$bFirstPicture)
							{
								$oProperty_Value->file_small = "small_{$sTargetFileName}";
								$oProperty_Value->file_small_name = '';
								$oProperty_Value->save();
							}
							else
							{
								$oShopItem->image_small = "small_{$sTargetFileName}";
								$oShopItem->setSmallImageSizes();
							}
						}

						$oShopItem->save() && $bFirstPicture = FALSE;
					}
				}

				// Добавляем значения для общих свойств всех товаров
				foreach($oItem->xpath('ЗначенияСвойств/ЗначенияСвойства') as $ItemPropertyValue)
				{
					$sPropertyValue = strval($ItemPropertyValue->Значение);
					$sPropertyGUID = strval($ItemPropertyValue->Ид);

					if(isset($this->aBaseProperties[$sPropertyGUID]))
					{
						switch($this->aBaseProperties[$sPropertyGUID])
						{
							case 'HOSTCMS_TITLE':
								$oShopItem->seo_title = $sPropertyValue;
							break;
							case 'HOSTCMS_DESCRIPTION':
								$oShopItem->seo_description = $sPropertyValue;
							break;
							case 'HOSTCMS_KEYWORDS':
								$oShopItem->seo_keywords = $sPropertyValue;
							break;
							case 'HOSTCMS_МЕТКИ':
								$oShopItem->applyTags($sPropertyValue);
							break;
							case 'YANDEX_MARKET':
								$oShopItem->yandex_market = $sPropertyValue;
							break;
							case 'ПРОДАВЕЦ':
								$oSeller = Core_Entity::factory('Shop', $this->iShopId)->Shop_Sellers->getByName($sPropertyValue);

								if(is_null($oSeller))
								{
									$oSeller = Core_Entity::factory('Shop_Seller');
									$oSeller->shop_id($this->iShopId)->name($sPropertyValue)->save();
								}

								$oShopItem->shop_seller_id = $oSeller->id;
							break;
							case 'ПРОИЗВОДИТЕЛЬ':
								$oProducer = Core_Entity::factory('Shop', $this->iShopId)->Shop_Producers->getByName($sPropertyValue);

								if(is_null($oProducer))
								{
									$oProducer = Core_Entity::factory('Shop_Producer');
									$oProducer->shop_id($this->iShopId)->name($sPropertyValue)->save();
								}

								$oShopItem->shop_producer_id = $oProducer->id;
							break;
							case 'АКТИВНОСТЬ':
								$oShopItem->active = $sPropertyValue;
							break;
						}

						$oShopItem->save();
					}
					else
					{
						$oShop_Item_Property_List = Core_Entity::factory('Shop_Item_Property_List', $this->iShopId);
						$oProperty = $oShop_Item_Property_List->Properties->getByGuid($sPropertyGUID);

						if(!is_null($oProperty))
						{
							$aPropertyValues = $oProperty->getValues($oShopItem->id, FALSE);

							$oProperty_Value = isset($aPropertyValues[0])
								? $aPropertyValues[0]
								: $oProperty->createNewValue($oShopItem->id);

							$oProperty_Value->setValue(isset($this->_aPropertyValues[$sPropertyValue])
								? $this->_aPropertyValues[$sPropertyValue]
								: $sPropertyValue);
							$oProperty_Value->save();
						}
					}
				}

				// Обрабатываем свойства/значения свойств для конкретно данного товара
				foreach($oItem->xpath('ХарактеристикиТовара/ХарактеристикаТовара | ЗначенияРеквизитов/ЗначениеРеквизита')
						  as $oItemProperty)
				{
					if(mb_strtoupper(strval($oItemProperty->Наименование)) == 'ВЕС')
					{
						$oShopItem->weight = Shop_Controller::instance()->convertPrice(strval($oItemProperty->Значение));
						$oShopItem->save();
					
						continue;
					}
				
					$oProperty = $this->_addItemProperty(array(
						'guid' => FALSE,
						'name' => strval($oItemProperty->Наименование),
						'group' => $oShopItem->shop_group_id
					));

					$aPropertyValues = $oProperty->getValues($oShopItem->id, FALSE);

					$oProperty_Value = isset($aPropertyValues[0])
						? $aPropertyValues[0]
						: $oProperty->createNewValue($oShopItem->id);

					$oProperty_Value->setValue(strval($oItemProperty->Значение));
					$oProperty_Value->save();
				}

				// Обрабатываем налоги
				foreach($oItem->xpath('СтавкиНалогов/СтавкаНалога') as $oTax)
				{
					$oShopTax = $this->_addTax($oTax);

					$oShopItem->shop_tax_id = $oShopTax->id;

					$oShopItem->save();
				}
			}
		}
		// Файл offers.xml
		elseif(count((array)$this->_oSimpleXMLElement->ПакетПредложений))
		{
			$packageOfProposals = $this->_oSimpleXMLElement->ПакетПредложений;

			// Обработка специальных цен
			foreach($packageOfProposals->xpath('ТипыЦен/ТипЦены') as $oPrice)
			{
				$oShopPrice = Core_Entity::factory('Shop', $this->iShopId)->Shop_Prices->getByGuid(strval($oPrice->Ид));

				if(is_null($oShopPrice))
				{
					$oShopPrice = Core_Entity::factory('Shop_Price');
					$oShopPrice->shop_id = $this->iShopId;
					$oShopPrice->guid = strval($oPrice->Ид);
					$oShopPrice->percent = 100;
				}

				$oShopPrice->name = strval($oPrice->Наименование);

				// Если это основная цена, обновляем информацию о налоге
				if(mb_strtoupper($oShopPrice->name) == $this->sShopDefaultPriceName)
				{
					$sTaxGUID = md5(mb_strtoupper($oPrice->Налог->Наименование));
					$oShopTax = Core_Entity::factory('Shop_Tax')->getByGuid($sTaxGUID);

					if(!is_null($oShopTax))
					{
						$iInSum = strval($oPrice->Налог->УчтеноВСумме);
						strtoupper($iInSum) == 'TRUE' ? $oShopTax->tax_is_included = 1 : $oShopTax->tax_is_included = 0;
						$oShopTax->save();
					}

					$this->sShopDefaultPriceGUID = $oShopPrice->guid;
				}
				else
				{
					$oShopPrice->save();
				}
			}

			// Обработка предложений
			foreach($packageOfProposals->xpath('Предложения/Предложение') as $oProposal)
			{
				$sItemGUID = strval($oProposal->Ид);

				if(strpos($sItemGUID, '#') !== FALSE)
				{
					$sItemGUID = explode('#', $sItemGUID);
					$sItemGUID = $sItemGUID[1];
				}

				$oShopItem = Core_Entity::factory('Shop', $this->iShopId)->Shop_Items->getByGuid($sItemGUID);

				if(!is_null($oShopItem))
				{
					// Товар найден, начинаем обновление
					$oShopItem->marking = strval($oProposal->Артикул);
					$oShopItem->name = strval($oProposal->Наименование);

					$oShopMeasure = Core_Entity::factory('Shop_Measure')->getByName(strval($oProposal->БазоваяЕдиница), FALSE);

					if(is_null($oShopMeasure))
					{
						$oShopMeasure = Core_Entity::factory('Shop_Measure');
						$oShopMeasure->name = strval($oProposal->БазоваяЕдиница);
						$oShopMeasure->description = strval(Core_Array::get($oProposal->БазоваяЕдиница->attributes(), 'НаименованиеПолное'));
						$oShopMeasure->save();
					}

					$oShopItem->shop_measure_id = $oShopMeasure->id;

					foreach($oProposal->xpath('Цены/Цена') as $oPrice)
					{
						// Ищем цену
						$oShopPrice = Core_Entity::factory('Shop_Price')->getByGuid(strval($oPrice->ИдТипаЦены));

						if(!is_null($oShopPrice) && $this->sShopDefaultPriceGUID != strval($oPrice->ИдТипаЦены))
						{
							$oShop_Item_Price = Core_Entity::factory('Shop_Item', $oShopItem->id)->Shop_Item_Prices->getByPriceId($oShopPrice->id);

							if(is_null($oShop_Item_Price))
							{
								$oShop_Item_Price = Core_Entity::factory('Shop_Item_Price');
								$oShop_Item_Price->shop_item_id = $oShopItem->id;
								$oShop_Item_Price->shop_price_id = $oShopPrice->id;
							}

							$itemPrice = strval($oPrice->ЦенаЗаЕдиницу);

							// Валюта товара
							$baseCurrencyNode = $oProposal->xpath("Цены/Цена[ИдТипаЦены='{$this->sShopDefaultPriceGUID}']");

							if(isset($baseCurrencyNode[0]))
							{
								$oItem_Shop_Currency = Core_Entity::factory('Shop_Currency')->getByLike(strval($baseCurrencyNode[0]->Валюта), FALSE);
								// Валюта спеццены
								$oPrice_Currency = Core_Entity::factory('Shop_Currency')->getByLike(strval($oPrice->Валюта), FALSE);

								if ($oItem_Shop_Currency->exchange_rate && $oPrice_Currency->exchange_rate)
								{
									$currencyCoefficient = Shop_Controller::instance()->getCurrencyCoefficientInShopCurrency($oPrice_Currency, $oItem_Shop_Currency);
			
									$itemPrice *= $currencyCoefficient;
								}
							}

							$oShop_Item_Price->value = $itemPrice;

							$oShopItem->add($oShop_Item_Price);
						}
						elseif($this->sShopDefaultPriceGUID == strval($oPrice->ИдТипаЦены))
						{
							$oShop_Currency = Core_Entity::factory('Shop_Currency')->getByLike(strval($oPrice->Валюта), FALSE);

							if(is_null($oShop_Currency))
							{
								$oShop_Currency = Core_Entity::factory('Shop_Currency');
								$oShop_Currency->name = strval($oPrice->Валюта);
								$oShop_Currency->exchange_rate = 1;
							}

							$oShopItem->price = strval($oPrice->ЦенаЗаЕдиницу);
							$oShopItem->add($oShop_Currency);
						}
					}

					foreach($oProposal->xpath('Количество') as $oCount)
					{
						// Ищем главный склад
						$oWarehouse = Core_Entity::factory('Shop', $this->iShopId)->Shop_Warehouses->getByDefault("1", FALSE);

						if(is_null($oWarehouse))
						{
							// Склад не обнаружен
							$oWarehouse = Core_Entity::factory('Shop_Warehouse');
							$oWarehouse->name = Core::_("Shop_Warehouse.warehouse_default_name");
							$oWarehouse->active = 1;
							$oWarehouse->default = 1;
							$oWarehouse->shop_id = $this->iShopId;
							$oWarehouse->save();
						}

						$oShop_Warehouse_Item = $oWarehouse->Shop_Warehouse_Items->getByShopItemId($oShopItem->id, FALSE);

						if(is_null($oShop_Warehouse_Item))
						{
							$oShop_Warehouse_Item = Core_Entity::factory('Shop_Warehouse_Item')
										->shop_warehouse_id($oWarehouse->id)
										->shop_item_id($oShopItem->id);
						}

						$oShop_Warehouse_Item->count(intval($oCount))->save();
					}

					$this->_aReturn['updateItemCount']++;
				}
			}
		}

		Core_Event::notify('Shop_Item_Import_Cml_Controller.onAfterImport', $this);
		
		return $this->_aReturn;
	}
}
