<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Online shop.
 *
 * @package HostCMS 6\Shop
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
class Shop_Item_Property_List_Model extends Shop_Model
{
	protected $_tableName = 'shops';
	protected $_modelName = 'shop_item_property_list';

	public $changeFilename = TRUE;
	public $watermarkFilePath = '';

	/**
	 * One-to-many or many-to-many relations
	 * @var array
	 */
	protected $_hasMany = array(
		'property' => array(
			'through' => 'shop_item_property',
			'foreign_key' => 'shop_id',
			'dependent_key' => 'property_id'
		),
		'property_dir' => array(
			'through' => 'shop_item_property_dir',
			'foreign_key' => 'shop_id',
			'dependent_key' => 'property_dir_id'
		)
	);

	/**
	 * Constructor.
	 */
	public function __construct($id = NULL)
	{
		parent::__construct($id);

		$this->changeFilename = $this->change_filename;
		$this->watermarkFilePath = $this->getWatermarkFilePath();
	}

	/**
	 * Get large image max width
	 * @return int
	 */
	public function getLargeImageMaxWidth()
	{
		return $this->image_large_max_width;
	}

	/**
	 * Get large image max height
	 * @return int
	 */
	public function getLargeImageMaxHeight()
	{
		return $this->image_large_max_height;
	}

	/**
	 * Get small image max width
	 * @return int
	 */
	public function getSmallImageMaxWidth()
	{
		return $this->image_small_max_width;
	}

	/**
	 * Get small image max height
	 * @return int
	 */
	public function getSmallImageMaxHeight()
	{
		return $this->image_small_max_height;
	}

	public function getDirHref(Core_Entity $object)
	{
		return $object->getItemHref();
	}

	public function getDirPath(Core_Entity $object)
	{
		return $object->getItemPath();
	}

	public function createPropertyDir(Core_Entity $object)
	{
		$object->createDir();
		return $this;
	}

	public function getLargeFileName(Core_Entity $object, $oFileValue, $originalFileName)
	{
		return 'shop_property_file_' . $object->id . '_' . $oFileValue->id . '.' . Core_File::getExtension($originalFileName);
	}

	public function getSmallFileName(Core_Entity $object, $oFileValue, $originalFileName)
	{
		return 'small_shop_property_file_' . $object->id . '_' . $oFileValue->id . '.' . Core_File::getExtension($originalFileName);
	}

	/**
	 * Получение свойств товара, доступных группе $shop_group_id
	 * @param int $shop_group_id идентификатор группы
	 * @return array
	 */
	public function getPropertiesForGroup($shop_group_id)
	{
		$oProperties = $this->Properties;
		if ($shop_group_id !== FALSE)
		{
			$oProperties
				->queryBuilder()
				->join('shop_item_property_for_groups', 'shop_item_property_for_groups.shop_item_property_id', '=', 'shop_item_properties.id')
				->where('shop_item_property_for_groups.shop_id', '=', $this->id)
				->where('shop_item_property_for_groups.shop_group_id', '=', $shop_group_id);
		}

		return $oProperties->findAll();
	}
}