<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Online shop.
 *
 * @package HostCMS 6\Shop
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
abstract class Shop_Payment_System_Handler
{
	static public function factory(Shop_Payment_System_Model $oShop_Payment_System_Model)
	{
		require_once($oShop_Payment_System_Model->getPaymentSystemFilePath());

		$name = 'Shop_Payment_System_Handler' . $oShop_Payment_System_Model->id;
		if (class_exists($name))
		{
			return new $name($oShop_Payment_System_Model);
		}
		return NULL;
	}

	protected $_orderParams = NULL;
	public function orderParams($orderParams)
	{
		$this->_orderParams = $orderParams + array(
			'invoice' => NULL,
			'coupon_text' => NULL,
		);
		return $this;
	}

	protected $_Shop_Payment_System_Model = NULL;

	/**
	 * Constructor.
	 */
	public function __construct(Shop_Payment_System_Model $oShop_Payment_System_Model)
	{
		$this->_Shop_Payment_System_Model = $oShop_Payment_System_Model;
	}

	/**
	 * Executes the business logic.
	 */
	public function execute()
	{
		Core_Session::start();

		!isset($_SESSION['last_order_id']) && $_SESSION['last_order_id'] = 0;

		// Если заказ еще не был оформлен
		if ($_SESSION['last_order_id'] == 0)
		{
			// Оформить новый заказ
			$this->_processOrder();

			$_SESSION['last_order_id'] = $this->_shopOrder->id;
		}
		else
		{
			$this->shopOrder(
				Core_Entity::factory('Shop_Order', intval($_SESSION['last_order_id']))
			);
		}

		return $this;
	}

	/**
	 * Объект заказа до изменения.
	 */
	protected $_shopOrderBeforeAction = NULL;
	public function shopOrderBeforeAction(Shop_Order_Model $oShopOrderBeforeAction)
	{
		$this->_shopOrderBeforeAction = $oShopOrderBeforeAction;
		return $this;
	}

	/**
	 * Объект заказа
	 */
	protected $_shopOrder = NULL;
	public function shopOrder(Shop_Order_Model $oShop_Order)
	{
		$this->_shopOrder = $oShop_Order;
		return $this;
	}

	/**
	 * Создание нового заказа на основе данных, указанных в orderParams
	 */
	protected function _processOrder()
	{
		if (is_null($this->_orderParams))
		{
			throw new Core_Exception('orderParams is empty.');
		}

		$oShop = $this->_Shop_Payment_System_Model->Shop;

		$this->_shopOrder = $oShop_Order = Core_Entity::factory('Shop_Order');
		$oShop_Order->shop_country_id = $this->_orderParams['shop_country_id'];
		$oShop_Order->shop_country_location_id = $this->_orderParams['shop_country_location_id'];
		$oShop_Order->shop_country_location_city_id = $this->_orderParams['shop_country_location_city_id'];
		$oShop_Order->shop_country_location_city_area_id = $this->_orderParams['shop_country_location_city_area_id'];
		$oShop_Order->postcode = $this->_orderParams['postcode'];
		$oShop_Order->address = $this->_orderParams['address'];
		$oShop_Order->surname = $this->_orderParams['surname'];
		$oShop_Order->name = $this->_orderParams['name'];
		$oShop_Order->patronymic = $this->_orderParams['patronymic'];
		$oShop_Order->company = $this->_orderParams['company'];
		$oShop_Order->phone = $this->_orderParams['phone'];
		$oShop_Order->fax = $this->_orderParams['fax'];
		$oShop_Order->email = $this->_orderParams['email'];
		$oShop_Order->description = $this->_orderParams['description'];
		$oShop_Order->shop_delivery_condition_id = $this->_orderParams['shop_delivery_condition_id'];
		$oShop_Order->shop_payment_system_id = $this->_orderParams['shop_payment_system_id'];
		$oShop_Order->shop_currency_id = $oShop->shop_currency_id;

		if (Core::moduleIsActive('siteuser'))
		{
			$oSiteuser = Core_Entity::factory('Siteuser')->getCurrent();
			$oSiteuser && $oShop_Order->siteuser_id = $oSiteuser->id;
		}

		$bInvoice = strlen($this->_orderParams['invoice']) > 0;
		$bInvoice && $oShop_Order->invoice = $this->_orderParams['invoice'];

		$oShop->add($oShop_Order);

		$this->shopOrder($oShop_Order);

		// Если не установлен модуль пользователей сайта - записываем в сессию
		// идентификатор вставленного заказа, чтобы далее можно было посмотреть квитаницию
		// об оплате или счет.
		if (!Core::moduleIsActive('siteuser'))
		{
			$_SESSION['order_' . $oShop_Order->id] = TRUE;
		}

		!$bInvoice && $oShop_Order->invoice($oShop_Order->id)->save();

		$quantity = 0;
		$amount = 0;

		$Shop_Cart_Controller = Shop_Cart_Controller::instance();

		$aShop_Cart = $Shop_Cart_Controller->getAll($oShop);
		foreach ($aShop_Cart as $oShop_Cart)
		{
			if ($oShop_Cart->postpone == 0)
			{
				$quantity += $oShop_Cart->quantity;

				$oShop_Order_Item = Core_Entity::factory('Shop_Order_Item');
				$oShop_Order_Item->quantity = $oShop_Cart->quantity;
				$oShop_Order_Item->shop_item_id = $oShop_Cart->shop_item_id;
				$oShop_Order_Item->shop_warehouse_id = $oShop_Cart->shop_warehouse_id;

				// Prices
				$oShop_Item_Controller = new Shop_Item_Controller();
				if (Core::moduleIsActive('siteuser'))
				{
					$oSiteuser = Core_Entity::factory('Siteuser')->getCurrent();
					$oSiteuser && $oShop_Item_Controller->siteuser($oSiteuser);
				}
				$oShop_Item_Controller->count($oShop_Cart->quantity);

				$aPrices = $oShop_Item_Controller->getPrices($oShop_Cart->Shop_Item);
				$amount += $aPrices['price_discount'] * $oShop_Cart->quantity;
				$oShop_Order_Item->price = $aPrices['price_discount'] - $aPrices['tax'];
				$oShop_Order_Item->rate = $aPrices['rate'];

				$oShop_Item = Core_Entity::factory('Shop_Item')->find($oShop_Cart->shop_item_id);
				$oShop_Order_Item->name = !is_null($oShop_Item) ? $oShop_Item->name : '';
				$oShop_Order_Item->marking = !is_null($oShop_Item) ? $oShop_Item->marking : '';

				$oShop_Order->add($oShop_Order_Item);

				// Delete item from cart
				$Shop_Cart_Controller
					->shop_item_id($oShop_Cart->shop_item_id)
					->delete();
			}
		}

		if ($amount > 0)
		{
			// Скидки от суммы заказа
			$oShop_Purchase_Discount_Controller = new Shop_Purchase_Discount_Controller($oShop);
			$oShop_Purchase_Discount_Controller
				->amount($amount)
				->quantity($quantity)
				->couponText($this->_orderParams['coupon_text'])
				;

			//$totalDiscount = 0;
			$aShop_Purchase_Discounts = $oShop_Purchase_Discount_Controller->getDiscounts();
			foreach ($aShop_Purchase_Discounts as $oShop_Purchase_Discount)
			{
				$oShop_Order_Item = Core_Entity::factory('Shop_Order_Item');
				$oShop_Order_Item->name = $oShop_Purchase_Discount->name;
				$oShop_Order_Item->quantity = 1;
				$oShop_Order_Item->price = -1 * $oShop_Purchase_Discount->getDiscountAmount();

				$oShop_Order->add($oShop_Order_Item);
				//$totalDiscount += $oShop_Purchase_Discount->getDiscountAmount();
			}
		}

		// Добавляем стоимость доставки как отдельный товар
		if ($this->_orderParams['shop_delivery_condition_id'])
		{
			$oShop_Delivery_Condition = Core_Entity::factory('Shop_Delivery_Condition', $this->_orderParams['shop_delivery_condition_id']);

			$oShop_Order_Item = Core_Entity::factory('Shop_Order_Item');
			$oShop_Order_Item->name = Core::_('Shop_Delivery.delivery', $oShop_Delivery_Condition->Shop_Delivery->name);
			$oShop_Order_Item->quantity = 1;

			$aPrice = $oShop_Delivery_Condition->getPriceArray();
			$oShop_Order_Item->rate = $aPrice['rate'];
			$oShop_Order_Item->price = $aPrice['price'] + $aPrice['tax'];
			$oShop_Order_Item->marking = !is_null($oShop_Delivery_Condition->marking)
				? $oShop_Delivery_Condition->marking
				: '';
			$oShop_Order_Item->type = 1;
			$oShop_Order->add($oShop_Order_Item);
		}

		return $this;
	}

	/**
	 * XSL данных о заказе
	 */
	protected $_xsl = NULL;
	public function xsl(Xsl_Model $oXsl)
	{
		$this->_xsl = $oXsl;
		return $this;
	}

	public function getInvoice()
	{
		return $this->_processXml();
	}

	public function getNotification()
	{
		return $this->_processXml();
	}

	public function printInvoice()
	{
		echo $this->getInvoice();
		return $this;
	}

	public function printNotification()
	{
		echo $this->getNotification();
		return $this;
	}

	protected function _processXml()
	{
		$oShop = $this->_shopOrder->Shop->clearEntities();

		$oShop
			->addEntity($oShop->Shop_Company)
			->addEntity(
				$oShop->Site->clearEntities()->showXmlAlias()
			)
			->addEntity(
				$this->_shopOrder->clearEntities()
					->showXmlCurrency(TRUE)
					->showXmlCountry(TRUE)
					->showXmlItems(TRUE)
					->showXmlDelivery(TRUE)
					->showXmlPaymentSystem(TRUE)
					->showXmlOrderStatus(TRUE)
			);

		$sXml = $oShop->getXml();

		//echo "<pre>" . htmlspecialchars($sXml) . "</pre>";
		$return = Xsl_Processor::instance()
			->xml($sXml)
			->xsl($this->_xsl)
			->process();

		$this->_shopOrder->clearEntities();

		return $return;
	}

	/**
	 * XSL письма администратору о заказе
	 */
	protected $_xslAdminMail = NULL;
	public function xslAdminMail(Xsl_Model $oXsl)
	{
		$this->_xslAdminMail = $oXsl;
		return $this;
	}

	/**
	 * XSL письма пользователю о заказе
	 */
	protected $_xslSiteuserMail = NULL;
	public function xslSiteuserMail(Xsl_Model $oXsl)
	{
		$this->_xslSiteuserMail = $oXsl;
		return $this;
	}

	/**
	 * Content-type письма администратору о заказе
	 */
	protected $_adminMailContentType = 'text/html';
	public function adminMailContentType($contentType)
	{
		$this->_adminMailContentType = $contentType;
		return $this;
	}

	/**
	 * Content-type письма пользователю о заказе
	 */
	protected $_siteuserMailContentType = 'text/html';
	public function siteuserMailContentType($contentType)
	{
		$this->_siteuserMailContentType = $contentType;
		return $this;
	}

	/**
	 * Тема письма администратору о заказе
	 */
	protected $_adminMailSubject = NULL;
	public function adminMailSubject($subject)
	{
		$this->_adminMailSubject = $subject;
		return $this;
	}

	/**
	 * Тема письма пользователю о заказе
	 */
	protected $_siteuserMailSubject = NULL;
	public function siteuserMailSubject($subject)
	{
		$this->_siteuserMailSubject = $subject;
		return $this;
	}

	public function getSiteuserEmail()
	{
		return Core_Mail::instance();
	}

	public function getAdminEmail()
	{
		return Core_Mail::instance();
	}

	public function setXSLs()
	{
		$oShopOrder = $this->_shopOrder;
		$oStructure = $oShopOrder->Shop->Structure;
		$libParams = $oStructure->Lib->getDat($oStructure->id);

		$this->xslAdminMail(
			Core_Entity::factory('Xsl')->getByName(
				Core_Array::get($libParams, 'orderAdminNotificationXsl')
			)
		)
		->xslSiteuserMail(
			Core_Entity::factory('Xsl')->getByName(
				Core_Array::get($libParams, 'orderUserNotificationXsl')
			)
		);

		return $this;
	}

	/**
	 * Send emails about order
	 */
	public function send()
	{
		if (is_null($this->_shopOrder))
		{
			throw new Core_Exception('send(): hopOrder is empty.');
		}

		$oShopOrder = $this->_shopOrder;
		$oShop = $oShopOrder->Shop;

		// Проверяем необходимость отправить письмо администратору
		if ($oShop->send_order_email_admin)
		{
			$oCore_Mail_Admin = $this->getAdminEmail();
			$this->sendAdminEmail($oCore_Mail_Admin);
		}

		if ($oShop->send_order_email_user)
		{
			$oCore_Mail_Siteuser = $this->getSiteuserEmail();
			$this->sendSiteuserEmail($oCore_Mail_Siteuser);
		}

		return $this;
	}

	public function sendAdminEmail(Core_Mail $oCore_Mail)
	{
		$oShopOrder = $this->_shopOrder;
		$oShop = $oShopOrder->Shop;

		// Адрес "ОТ КОГО" для администратора
		$from = $oShop->getFirstEmail();

		$this->xsl($this->_xslAdminMail);
		$sInvoice = $this->_processXml();
		$sInvoice = str_replace(">", ">\n", $sInvoice);

		// Тема письма администратору
		$date_str = Core_Date::sql2datetime($oShopOrder->datetime);
		$admin_subject = !is_null($this->_adminMailSubject)
			? $this->_adminMailSubject
			: Core::_('Shop_Order.shop_order_admin_subject', $oShopOrder->invoice, $oShop->name, $date_str);

		$oCore_Mail
			->from($from)
			->subject($admin_subject)
			->message($sInvoice)
			->contentType($this->_adminMailContentType)
			->header('X-HostCMS-Reason', 'Order')
			->header('Precedence', 'bulk');

		$aEmails = trim($oShop->email) != ''
			? explode(',', $oShop->email)
			: array(EMAIL_TO);

		foreach ($aEmails as $sEmail)
		{
			if (Core_Valid::email($sEmail))
			{
				$oCore_Mail->to($sEmail)->send();

				// Anti spam filter
				sleep(1);
			}
		}

		return $this;
	}

	protected function _attachDigitalItems(Core_Mail $oCore_Mail)
	{
		$aShop_Order_Items = $this->_shopOrder->Shop_Order_Items->findAll(FALSE);
		foreach ($aShop_Order_Items as $oShop_Order_Item)
		{
			// Digital items
			$aShop_Order_Item_Digitals = $oShop_Order_Item->Shop_Order_Item_Digitals->findAll();
			foreach ($aShop_Order_Item_Digitals as $oShop_Order_Item_Digital)
			{
				$oShop_Item_Digital = $oShop_Order_Item_Digital->Shop_Item_Digital;

				if ($oShop_Item_Digital->filename != '' && is_file($oShop_Item_Digital->getFullFilePath()))
				{
					$oCore_Mail->attach(array(
					 'filepath' => $oShop_Item_Digital->getFullFilePath(),
					 'filename' => $oShop_Item_Digital->filename,
					 ));
				}
			}
		}
		return $this;
	}

	public function sendSiteuserEmail(Core_Mail $oCore_Mail)
	{
		$oShopOrder = $this->_shopOrder;
		$oShop = $oShopOrder->Shop;

		$to = $oShopOrder->email;

		if (Core_Valid::email($to))
		{
			// Адрес "ОТ КОГО" для пользователя
			$from = $oShop->getFirstEmail();

			$this->xsl($this->_xslSiteuserMail);
			$sInvoice = $this->_processXml();
			$sInvoice = str_replace(">", ">\n", $sInvoice);

			// Тема письма пользователю
			$user_subject = !is_null($this->_siteuserMailSubject)
				? $this->_siteuserMailSubject
				: "Информация о заказе";

			// Attach digitals items
			if ($this->_shopOrder->paid == 1 && $this->_shopOrder->Shop->attach_digital_items == 1)
			{
				$this->_attachDigitalItems($oCore_Mail);
			}

			$oCore_Mail
				->from($from)
				->to($to)
				->subject($user_subject)
				->message($sInvoice)
				->contentType($this->_adminMailContentType)
				->header('X-HostCMS-Reason', 'OrderConfirm')
				->header('Precedence', 'bulk')
				->send();
		}

		return $this;
	}

	/**
	 * Уведомление об операциях с заказом
	 * @param string $mode режим изменения:
	 * - edit - редактирование заказа
	 */
	public function changedOrder($mode)
	{
		if (in_array($mode, array('changeStatusPaid', 'edit')))
		{
			if ($this->_shopOrderBeforeAction->paid != $this->_shopOrder->paid)
			{
				// Установка XSL-шаблонов в соответствии с настройками в узле структуры
				$this->setXSLs();

				// Отправка писем клиенту и пользователю
				$this->send();
			}
		}

		return $this;
	}
}