<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Sites.
 *
 * @package HostCMS 6\Site
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
class Site_Controller_Template extends Core_Servant_Properties
{
	/**
	 * Allowed object properties
	 * @var array
	 */
	protected $_allowedProperties = array(
		'server',
		'chmodFile',
		'templatePath',
		'templateFilename',
		'templateSelectedFilename',
	);

	/**
	 * Constructor.
	 */
	public function __construct()
	{
		parent::__construct();
		$this->server = 'http://www.hostcms.ru';
		$this->chmodFile = 0644;
		$this->templatePath = CMS_FOLDER . TMP_DIR;
		$this->templateFilename = 'templates.xml';
		$this->templateSelectedFilename = 'template.xml';
	}

	/**
	 * The singleton instances.
	 * @var mixed
	 */
	static public $instance = NULL;

	/**
	 * Register an existing instance as a singleton.
	 * @return object
	 */
	static public function instance()
	{
		if (is_null(self::$instance))
		{
			self::$instance = new self();
		}

		return self::$instance;
	}

	public function getSelectedTemplateXml()
	{
		$template_content_filepath = $this->templatePath . $this->templateSelectedFilename;

		if(is_file($template_content_filepath) && ($sTemplateContentXml = Core_File::read($template_content_filepath)))
		{
			if($aTemplateContentXml = Core_XML::xml2array($sTemplateContentXml))
			{
				return $aTemplateContentXml;
			}
		}

		return FALSE;
	}

	public function getTemplateXml()
	{
		$template_filepath = $this->templatePath . $this->templateFilename;

		try
		{
			$Core_Http = Core_Http::instance()
				->url($this->server . '/hostcms/templates/xml/')
				->port(80)
				->timeout(5)
				->execute();

			$sXmlTemplates = $Core_Http->getBody();
		}
		catch (Exception $e)
		{
			Core_Message::show($e->getMessage(), 'error');
			$sXmlTemplates = NULL;
		}

		if ($sXmlTemplates)
		{
			try
			{
				Core_File::write($template_filepath, $sXmlTemplates, $this->chmodFile);
			}
			catch (Exception $e)
			{
				Core_Message::show($e->getMessage(), 'error');
			}
		}
		elseif (file_exists($template_filepath))
		{
			$sXmlTemplates = Core_File::read($template_filepath);
		}

		$sXmlTemplates = str_replace('&nbsp;', ' ', $sXmlTemplates);

		return $sXmlTemplates;
	}

	public function loadTemplatesXml()
	{
		// Массив по данным templates.xml
		$aTemplatesXml = $this->getTemplateXml();

		if (!$aTemplatesXml)
		{
			Core_Message::show(
				Core::_('install.write_error', $this->templatePath . $this->templateFilename)
			, 'error');
		}

		return simplexml_load_string($aTemplatesXml);
	}
	
	public function getTemplateXmlArray()
	{
		$sXmlTemplates = $this->getTemplateXml();
		return Core_XML::xml2array($sXmlTemplates);
	}

	public function getFields(array $array)
	{
		$return = array();

		// цикл по дереву 'fields'
		foreach ($array as $aFieldsValue)
		{
			$tmp = array(
				'Name' => '',
				'Type' => $aFieldsValue['attr']['type'],
				'Value' => '',
				'Macros' => '',
				'Path' => '',
				'Extension' => '',
				'MaxWidth' => 0,
				'MaxHeight' => 0,
				'ListValue' => array()
			);

			foreach ($aFieldsValue['children'] as $aInputValues)
			{
				if (isset($aInputValues['name']))
				{
					switch ($aInputValues['name'])
					{
						case 'name':
							$tmp['Name'] = $aInputValues['value'];
						break;

						case 'value':
							$tmp['Value'] = $aInputValues['value'];

							// Значения для списка
							if (isset($aInputValues['children']))
							{
								foreach ($aInputValues['children'] as $aListValues)
								{
									if (isset($aListValues['attr']['value']))
									{
										$tmp['ListValue'][$aListValues['attr']['value']]
										= $aListValues['value'];
									}
								}
							}
						break;

						case 'macros':
							$tmp['Macros'] = $aInputValues['value'];
						break;

						case 'path':
							$tmp['Path'] = $aInputValues['value'];
						break;

						case 'extension':
							$tmp['Extension'] = $aInputValues['value'];
						break;

						case 'max_width':
							$tmp['MaxWidth'] = intval($aInputValues['value']);
						break;

						case 'max_height':
							$tmp['MaxHeight'] = intval($aInputValues['value']);
						break;
					}
				}
			}

			$return[] = $tmp;
		}

		return $return;
	}

	public function macroReplace($str, $aReplace)
	{
		if (count($aReplace) > 0)
		{
			foreach ($aReplace as $key => $value)
			{
				$str = str_replace($key, $value, $str);
			}
		}

		return $str;
	}

	public function loadFile($filename, $aReplace = array())
	{
		$filecontent = Core_File::read($filename);
		if ($filecontent)
		{
			$filecontent = $this->macroReplace($filecontent, $aReplace);
		}
		return $filecontent;
	}

	/**
	 * Заменяет макросы в уже существующем файле
	 *
	 *  @param $filename путь к файлу
	 *  @param $aReplace массив замен
	 */
	public function replaceFile($filename, $aReplace = array())
	{
		file_put_contents($filename, $this->loadFile($filename, $aReplace));
		return $this;
	}

	/**
	* Определение расширения файла по его названию без расширения
	*
	* @param $path_file_without_extension - путь к файлу без расширения
	*/
	public function getFileExtension($path_file_without_extension)
	{
		clearstatcache();

		// Директория, в которой находится файл
		$dirname = dirname($path_file_without_extension);

		// Имя файла
		$filename = basename($path_file_without_extension);

		$extension = '';

		if (is_dir($dirname) && !is_link($dirname) && $handle = @opendir($dirname))
		{
			// Шаблон сравнения
			$str = "/^" . $filename . ".([[:alpha:]]*)$/";

			// Просматриваем файлы директории
			while (false !== ($file = readdir($handle)))
			{
				if (preg_match($str, $file , $regs) && Core_Type_Conversion::toStr($regs[1])!= '')
				{
					$extension = $regs[1];
					break;
				}
			}
			closedir($handle);

			return $extension;
		}
		return NULL;
	}
}