<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Sites.
 *
 * @package HostCMS 6\Site
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
class Site_Model extends Core_Entity
{
	public $img = 1;
	public $domains = 1;

	/**
	 * Forbidden tags. If list of tags is empty, all tags will show.
	 * @var array
	 */
	protected $_forbiddenTags = array(
		'robots',
		'key',
		'html_cache_use',
		'html_cache_with',
		'html_cache_without',
		'html_cache_clear_probability',
	);

	/**
	 * One-to-many or many-to-many relations
	 * @var array
	 */
	protected $_hasMany = array(
		'affiliate_plan' => array(),
		'advertisement' => array(),
		'advertisement_group' => array(),
		'counter' => array(),
		'counter_page' => array(),
		'document' => array(),
		'document_dir' => array(),
		'document_status' => array(),
		'informationsystem' => array(),
		'informationsystem_dir'	=> array(),
		'list' => array(),
		'list_dir' => array(),
		'maillist' => array(),
		'poll_group' => array(),
		'search_log' => array(),
		'search_page' => array(),
		'seo' => array(),
		'seo_query' => array(),
		'shop' => array(),
		'shop_dir' => array(),
		'siteuser' => array(),
		'siteuser_group' => array(),
		'site_alias' => array(),
		'structure' => array(),
		'structure_menu' => array(),
		'template' => array(),
		'template_dir' => array(),
		'user_module' => array(),
		'user_group' => array(),
		'user_group_action_access' => array(),
		'helpdesk' => array(),
		'forum' => array(),
		'form' => array()
	);

	protected $_sorting = array(
		'sites.sorting' => 'ASC',
		'sites.name' => 'ASC'
	);

	/**
	 * List of preloaded values
	 * @var array
	 */
	protected $_preloadValues = array(
		'name' => '',
		'active' => 1,
		'coding' => 'UTF-8',
		'sorting' => 0,
		'locale' => 'ru_RU.utf8',
		'max_size_load_image' => 70,
		'max_size_load_image_big' => 300,
		'send_attendance_report' => 1,
		'chmod' => '0755',
		'files_chmod' => '0644',
		'date_format' => '%d.%m.%Y',
		'date_time_format' => '%d.%m.%Y %H:%M:%S',
		'error' => 'E_ALL',
		'error404' => 0,
		'error403' => 0,
		'robots' => "User-agent: *\nDisallow: /admin",
		'closed' => 0,
		'safe_email' => 1,
		'html_cache_use' => 0,
		'html_cache_with' => '/*',
		'html_cache_clear_probability' => 10000,
		'uploaddir' => 'upload/',
		'nesting_level' => 3,
		'timezone' => ''
	);

	/**
	 * Constructor.
	 */
	public function __construct($id = NULL)
	{
		parent::__construct($id);

		if (is_null($id))
		{
			$oUserCurrent = Core_Entity::factory('User', 0)->getCurrent();
			$this->_preloadValues['user_id'] = is_null($oUserCurrent) ? 0 : $oUserCurrent->id;
		}
	}

	public function changeStatus()
	{
		$this->active = 1 - $this->active;
		return $this->save();
	}

	public function getByAlias($alias_name)
	{
		$oSiteAliases = Core_Entity::factory('Site_Alias');
		$oSiteAliases->queryBuilder()
			->join('sites', 'sites.id', '=', 'site_aliases.site_id')
			->where('site_aliases.name', 'LIKE', $alias_name)
			->where('sites.deleted', '=', 0)
			->limit(1);
		$aSiteAlias = $oSiteAliases->findAll();

		if (isset($aSiteAlias[0]))
		{
			return $aSiteAlias[0]->Site;
		}

		// Удаляем все переданные *. если они были
		$new_alias_name = $this->Site_Aliases->ReplaceMask($alias_name);

		// Если в переданном алиасе небыло *.
		if (mb_strpos($alias_name, '*.') === FALSE)
		{
			$new_alias_name = "*." . $alias_name;
			return $this->GetByAlias($new_alias_name);
		}
		// Если в пути осталась хоть одна точка
		elseif (mb_strpos($new_alias_name, '.') !== FALSE)
		{
			$new_alias_name = "*." . mb_substr($new_alias_name, mb_strpos($new_alias_name, '.') + 1);
			return $this->GetByAlias($new_alias_name);
		}

		return NULL;
	}

	/**
	 * Delete object from database
	 * @param mixed $primaryKey primary key for deleting object
	 * @return Core_Entity
	 */
	public function delete($primaryKey = NULL)
	{
		if (is_null($primaryKey))
		{
			$primaryKey = $this->getPrimaryKey();
		}
		$this->id = $primaryKey;

		// Advertisement
		if (Core::moduleIsActive('advertisement'))
		{
			$this->Advertisements->deleteAll(FALSE);
			$this->Advertisement_Groups->deleteAll(FALSE);
		}

		// Counter
		if (Core::moduleIsActive('counter'))
		{
			$this->Counters->deleteAll(FALSE);
			$this->Counter_Pages->deleteAll(FALSE);
		}

		// Documents
		$this->Documents->deleteAll(FALSE);
		$this->Document_Dirs->deleteAll(FALSE);
		$this->Document_Statuses->deleteAll(FALSE);

		// Information systems
		$this->Informationsystems->deleteAll(FALSE);
		$this->Informationsystem_Dirs->deleteAll(FALSE);

		// Lists
		if (Core::moduleIsActive('list'))
		{
			$this->Lists->deleteAll(FALSE);
			$this->List_Dirs->deleteAll(FALSE);
		}

		// Maillist
		if (Core::moduleIsActive('maillist'))
		{
			$this->Maillists->deleteAll(FALSE);
		}

		// Polls
		if (Core::moduleIsActive('poll'))
		{
			$this->Poll_Groups->deleteAll(FALSE);
		}

		// Helpdesks
		if (Core::moduleIsActive('helpdesk'))
		{
			$this->Helpdesks->deleteAll(FALSE);
		}

		// Forums
		if (Core::moduleIsActive('forum'))
		{
			$this->Forums->deleteAll(FALSE);
		}

		// Search
		if (Core::moduleIsActive('search'))
		{
			$this->Search_Logs->deleteAll(FALSE);
			$this->Search_Pages->deleteAll(FALSE);
		}

		$this->Seos->deleteAll(FALSE);
		$this->Seo_Queries->deleteAll(FALSE);

		// Shops
		$this->Shops->deleteAll(FALSE);
		$this->Shop_Dirs->deleteAll(FALSE);

		// Siteusers
		if (Core::moduleIsActive('siteuser'))
		{
			$this->Affiliate_Plans->deleteAll(FALSE);
			$this->Siteusers->deleteAll(FALSE);
			$this->Siteuser_Groups->deleteAll(FALSE);

			// Доп. св-ва пользователей сайта
			$oSiteuser_Property_List = Core_Entity::factory('Siteuser_Property_List', $this->id);
			$oSiteuser_Property_List->Properties->deleteAll(FALSE);
			$oSiteuser_Property_List->Property_Dirs->deleteAll(FALSE);
		}

		$this->Site_Aliases->deleteAll(FALSE);

		// Structures
		$this->Structures->deleteAll(FALSE);
		$this->Structure_Menus->deleteAll(FALSE);

		// Templates
		$this->Templates->deleteAll(FALSE);
		$this->Template_Dirs->deleteAll(FALSE);

		// Users
		$this->User_Groups->deleteAll(FALSE);
		$this->User_Group_Action_Accesses->deleteAll(FALSE);
		$this->User_Modules->deleteAll(FALSE);

		// Удаление доп. св-в структуры сайта
		$oStructure_Property_List = Core_Entity::factory('Structure_Property_List', $this->id);
		$oStructure_Property_List->Properties->deleteAll(FALSE);
		$oStructure_Property_List->Property_Dirs->deleteAll(FALSE);

		// Удаление директории с доп. св-вами структуры
		try
		{
			Core_File::deleteDir(CMS_FOLDER . $this->uploaddir . 'structure_' . intval($this->id));
		} catch (Exception $e) {};

		$this->deleteIcoFile();

		return parent::delete($primaryKey);
	}

	public function deleteIcoFile()
	{
		try
		{
			Core_File::delete($this->getIcoFilePath());
		} catch (Exception $e) {};

		return $this;
	}

	protected $_currentAlias = NULL;
	public function getCurrentAlias($bCache = TRUE)
	{
		if ($bCache && !is_null($this->_currentAlias))
		{
			return $this->_currentAlias;
		}

		$siteAlias = $this->Site_Aliases;

		$siteAlias->queryBuilder()
			// ->clear()
			->where('current', '=', 1)
			->limit(1);

		$aSiteAlias = $siteAlias->findAll();
		if (count($aSiteAlias) > 0)
		{
			$aSiteAlias[0]->name = $aSiteAlias[0]->alias_name_without_mask;

			$bCache && $this->_currentAlias = $aSiteAlias[0];

			return $aSiteAlias[0];
		}

		return NULL;
	}

	public function getFirstSite()
	{
		$this->queryBuilder()
			->clear()
			->orderBy('sorting', 'ASC')
			->limit(1);

		$result = $this->findAll();

		if (!empty($result))
		{
			return $result[0];
		}

		return NULL;
	}

	protected function _getFaviconPath()
	{
		return CMS_FOLDER . $this->uploaddir . "favicon";
	}

	protected function _createFaviconDir()
	{
		$uploaddir_path = $this->_getFaviconPath();

		if (!is_dir($uploaddir_path))
		{
			try
			{
				Core_File::mkdir($uploaddir_path);
			} catch (Exception $e) {};
		}
		return $this;
	}

	public function getIcoFilePath()
	{
		return $this->_getFaviconPath() . "/site_" . intval($this->id) . ".ico";
	}

	public function getIcoFileHref()
	{
		return '/' . $this->uploaddir . "favicon/site_" . intval($this->id) . ".ico";
	}

	public function saveIcoFile($fileSourcePath)
	{
		$this->save()->_createFaviconDir();
		Core_File::upload($fileSourcePath, $this->getIcoFilePath());

		// Delete loaded PNG-file
		if (is_file($this->getPngFilePath()))
		{
			try
			{
				Core_File::delete($this->getPngFilePath());
			} catch (Exception $e) {};
		}

		return $this;
	}

	public function getPngFilePath()
	{
		return $this->_getFaviconPath() . "/site_" . intval($this->id) . ".png";
	}

	public function getPngFileHref()
	{
		return '/' . $this->uploaddir . "favicon/site_" . intval($this->id) . ".png";
	}

	public function savePngFile($fileSourcePath)
	{
		$this->save()->_createFaviconDir();
		Core_File::upload($fileSourcePath, $this->getPngFilePath());

		// Delete loaded ICO-file
		if (is_file($this->getIcoFilePath()))
		{
			try
			{
				Core_File::delete($this->getIcoFilePath());
			} catch (Exception $e) {};
		}

		return $this;
	}

	/**
	 * Save object.
	 *
	 * @return Core_Entity
	 */
	public function save()
	{
		// Создание директории uploaddir
		if (strlen($this->uploaddir))
		{
			$uploaddir_path = CMS_FOLDER . $this->uploaddir;

			if (!is_dir($uploaddir_path))
			{
				try
				{
					Core_File::mkdir($uploaddir_path);
				} catch (Exception $e) {};
			}
		}

		return parent::save();
	}

	/**
	 * Copy object
	 * @return Core_Entity
	 */
	public function copy()
	{
		$newObject = parent::copy();

		try {
			Core_File::copy($this->getIcoFilePath(), $newObject->getIcoFilePath());
		}
		catch(Exception $e){}

		// WARNING !!! Добавить копирование множества сущностей, связанных с сайтом

		return $newObject;
	}

	public function getKeys()
	{
		$sKeys = trim(str_replace(array("\n", "\r", "\0", "\t", ), '', $this->key));
		return str_split($sKeys, 29);
	}

	protected $_showXmlAlias = FALSE;
	public function showXmlAlias($showXmlAlias = TRUE)
	{
		$this->_showXmlAlias = $showXmlAlias;
		return $this;
	}

	/**
	 * Get XML for entity and children entities
	 * @return string
	 */
	public function getXml()
	{
		if ($this->_showXmlAlias)
		{
			$oSite_Alias = $this->getCurrentAlias();
			!is_null($oSite_Alias) && $this->addEntity($oSite_Alias);
		}

		return parent::getXml();
	}
}