<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Показ навигационной цепочки структуры сайта.
 *
 * Доступные методы:
 * current($parentId) вывод потомков узла структуры $parentId
 * showProperties(TRUE) выводить значения дополнительных свойств, по умолчанию NULL
 *
 * <code>
 * $Structure_Controller_Breadcrumbs = new Structure_Controller_Breadcrumbs(
 * 		Core_Entity::factory('Site', 1)
 * 	);
 *
 * 	$Structure_Controller_Breadcrumbs
 * 		->xsl(
 * 			Core_Entity::factory('Xsl')->getByName('ХлебныеКрошки')
 * 		)
 * 		->show();
 * </code>
 *
 * @package HostCMS 6\Structure
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
class Structure_Controller_Breadcrumbs extends Core_Controller
{
	/**
	 * Allowed object properties
	 * @var array
	 */
	protected $_allowedProperties = array(
		'menu',
		'current',
		'showProperties',
		'showInformationsystem',
		'showShop',
		'cache',
	);

	protected $_aProperties = array();
	protected $_aProperty_Dirs = array();

	/**
	 * Constructor.
	 */
	public function __construct(Site_Model $oSite)
	{
		parent::__construct($oSite->clearEntities());

		$this->current = Core_Page::instance()->structure->id;
		$this->showInformationsystem = TRUE;
		$this->showShop = TRUE;
		$this->cache = TRUE;
	}

	protected $_Informationsystems = array();

	/**
	 * Show built data
	 * @return self
	 */
	public function show()
	{
		if ($this->cache && Core::moduleIsActive('cache'))
		{
			$oCore_Cache = Core_Cache::instance(Core::$mainConfig['defaultCache']);
			$inCache = $oCore_Cache->get($cacheKey = strval($this), $cacheName = 'structure_breadcrumbs');

			if (!is_null($inCache))
			{
				echo $inCache;
				return $this;
			}
		}

		$this->addEntity(
			Core::factory('Core_Xml_Entity')
				->name('current_structure_id')
				->value($this->current)
		);

		$aStructures = array();

		if (is_object(Core_Page::instance()->object))
		{
			if ($this->showInformationsystem && get_class(Core_Page::instance()->object) == 'Informationsystem_Controller_Show')
			{
				if (Core_Page::instance()->object->item)
				{
					$oInformationsystem_Item = Core_Entity::factory('Informationsystem_Item', Core_Page::instance()->object->item);

					$oInformationsystem_Item
						->clearEntities()
						->addForbiddenTag('url')
						->addEntity(
							Core::factory('Core_Xml_Entity')
								->name('link')
								->value(
									$oInformationsystem_Item->Informationsystem->Structure->getPath() . $oInformationsystem_Item->getPath()
								)
						)->addEntity(
							Core::factory('Core_Xml_Entity')
								->name('show')
								->value($oInformationsystem_Item->active)
						);

					$aStructures[] = $oInformationsystem_Item;
				}

				if (Core_Page::instance()->object->group)
				{
					$groupId = Core_Page::instance()->object->group;

					$aGroups = array();

					while ($groupId)
					{
						$oInformationsystem_Group = Core_Entity::factory('Informationsystem_Group', $groupId);

						$oInformationsystem_Group
							->clearEntities()
							->addForbiddenTag('url')
							->addEntity(
								Core::factory('Core_Xml_Entity')
									->name('link')
									->value(
										$oInformationsystem_Group->Informationsystem->Structure->getPath() . $oInformationsystem_Group->getPath()
									)
							)->addEntity(
								Core::factory('Core_Xml_Entity')
									->name('show')
									->value($oInformationsystem_Group->active)
							);

						$groupId = $oInformationsystem_Group->parent_id;
						$aGroups[] = $oInformationsystem_Group;
					}

					$aStructures = array_merge($aStructures, $aGroups);
				}
			}

			if ($this->showShop && get_class(Core_Page::instance()->object) == 'Shop_Controller_Show')
			{
				if (Core_Page::instance()->object->item)
				{
					$oShop_Item = Core_Entity::factory('Shop_Item', Core_Page::instance()->object->item);

					$oShop_Item
						->clearEntities()
						->addForbiddenTag('url')
						->addEntity(
							Core::factory('Core_Xml_Entity')
								->name('link')
								->value(
									$oShop_Item->Shop->Structure->getPath() . $oShop_Item->getPath()
								)
						)->addEntity(
							Core::factory('Core_Xml_Entity')
								->name('show')
								->value($oShop_Item->active)
						);

					$aStructures[] = $oShop_Item;
				}

				if (Core_Page::instance()->object->group)
				{
					$groupId = Core_Page::instance()->object->group;

					$aGroups = array();

					while ($groupId)
					{
						$oShop_Group = Core_Entity::factory('Shop_Group', $groupId);

						$oShop_Group
							->clearEntities()
							->addForbiddenTag('url')
							->addEntity(
								Core::factory('Core_Xml_Entity')
									->name('link')
									->value(
										$oShop_Group->Shop->Structure->getPath() . $oShop_Group->getPath()
									)
							)->addEntity(
								Core::factory('Core_Xml_Entity')
									->name('show')
									->value($oShop_Group->active)
							);

						$groupId = $oShop_Group->parent_id;
						$aGroups[] = $oShop_Group;
					}

					$aStructures = array_merge($aStructures, $aGroups);
				}
			}
		}

		$oStructure = Core_Entity::factory('Structure', $this->current)
			->clearEntities();

		do {
			$aStructures[] = $oStructure->clearEntities();
		} while($oStructure = $oStructure->getParent());

		$aStructures = array_reverse($aStructures);

		$object = $this;
		foreach ($aStructures as $oStructure)
		{
			$object->addEntity($oStructure);
			$object = $oStructure;
		}

		$oSite = $this->getEntity();

		// Показывать дополнительные свойства
		if ($this->showProperties)
		{
			$oStructure_Property_List = Core_Entity::factory('Structure_Property_List', $oSite->id);

			$aProperties = $oStructure_Property_List->Properties->findAll();
			foreach ($aProperties as $oProperty)
			{
				$this->_aProperties[$oProperty->property_dir_id][] = $oProperty;

				// Load all values for property
				$oProperty->loadAllValues();
			}

			$aProperty_Dirs = $oStructure_Property_List->Property_Dirs->findAll();
			foreach ($aProperty_Dirs as $oProperty_Dir)
			{
				$oProperty_Dir->clearEntities();
				$this->_aProperty_Dirs[$oProperty_Dir->parent_id][] = $oProperty_Dir;
			}

			$this->_addPropertyList(0, $this);
		}

		// Clear
		$this->_aProperty_Dirs = $this->_aProperties = array();

		echo $content = parent::get();
		$this->cache && Core::moduleIsActive('cache') && $oCore_Cache->set($cacheKey, $content, $cacheName);

		return $this;
	}

	protected function _addPropertyList($parent_id, $parentObject)
	{
		if (isset($this->_aProperty_Dirs[$parent_id]))
		{
			foreach ($this->_aProperty_Dirs[$parent_id] as $oProperty_Dir)
			{
				$parentObject->addEntity($oProperty_Dir);
				$this->_addPropertyList($oProperty_Dir->id, $oProperty_Dir);
			}
		}

		if (isset($this->_aProperties[$parent_id]))
		{
			$parentObject->addEntities($this->_aProperties[$parent_id]);
		}

		return $this;
	}
}
