<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Structure.
 *
 * @package HostCMS 6\Structure
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
class Structure_Property_List_Model extends Site_Model
{
	protected $_tableName = 'sites';
	protected $_modelName = 'structure_property_list';

	public $changeFilename = TRUE;
	public $watermarkFilePath = '';

	/**
	 * One-to-many or many-to-many relations
	 * @var array
	 */
	protected $_hasMany = array(
		'property' => array(
			'through' => 'structure_property',
			'foreign_key' => 'site_id',
			'dependent_key' => 'property_id'
		),
		'property_dir' => array(
			'through' => 'structure_property_dir',
			'foreign_key' => 'site_id',
			'dependent_key' => 'property_dir_id'
		)
	);

	/**
	 * Get large image max width
	 * @return int
	 */
	public function getLargeImageMaxWidth()
	{
		return $this->max_size_load_image;
	}

	/**
	 * Get large image max height
	 * @return int
	 */
	public function getLargeImageMaxHeight()
	{
		return $this->max_size_load_image;
	}

	/**
	 * Get small image max width
	 * @return int
	 */
	public function getSmallImageMaxWidth()
	{
		return $this->max_size_load_image_big;
	}

	/**
	 * Get small image max height
	 * @return int
	 */
	public function getSmallImageMaxHeight()
	{
		return $this->max_size_load_image_big;
	}
	
	public function getDirHref(Core_Entity $object)
	{
		return '/' . $object->getDirHref();
	}

	public function getDirPath(Core_Entity $object)
	{
		return $object->getDirPath();
	}

	public function createPropertyDir(Core_Entity $object)
	{
		$path = $this->getDirPath($object);
		if (!is_dir($path))
		{
			try
			{
				Core_File::mkdir($path, CHMOD, TRUE);
			} catch (Exception $e) {};
		}

		return $this;
	}

	public function getLargeFileName(Core_Entity $object, $oFileValue, $originalFileName)
	{
		return 'structure_property_image_' . $oFileValue->id . '.' . Core_File::getExtension($originalFileName);
	}

	public function getSmallFileName(Core_Entity $object, $oFileValue, $originalFileName)
	{
		return 'structure_property_small_image_' . $oFileValue->id . '.' . Core_File::getExtension($originalFileName);
	}
}