<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Templates.
 *
 * @package HostCMS 6\Template
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
class Template_Model extends Core_Entity
{
	public $img = 1;

	/**
	 * Belongs to relations
	 * @var array
	 */
	protected $_belongsTo = array(
		'template_dir' => array(),
		'template' => array(),
		'site' => array()
	);

	/**
	 * One-to-many or many-to-many relations
	 * @var array
	 */
	protected $_hasMany = array(
		'template' => array()
	);

	/**
	 * List of preloaded values
	 * @var array
	 */
	protected $_preloadValues = array(
		'sorting' => 0,
		'data_template_id' => 0
	);

	protected $_sorting = array(
		'templates.sorting' => 'ASC'
	);

	/**
	 * Constructor.
	 */
	public function __construct($id = NULL)
	{
		parent::__construct($id);

		if (is_null($id))
		{
			$oUserCurrent = Core_Entity::factory('User', 0)->getCurrent();
			$this->_preloadValues['user_id'] = is_null($oUserCurrent) ? 0 : $oUserCurrent->id;
		}
	}

	/**
	 * Executes the business logic.
	 */
	public function execute()
	{
		// Совместимость с HostCMS 5
		if (defined('USE_HOSTCMS_5') && USE_HOSTCMS_5)
		{
			$kernel = & singleton('kernel');
		}

		include $this->getTemplateFilePath();
		return $this;
	}

	public function getBySiteId($site_id)
	{
		$this->queryBuilder()
			//->clear()
			->where('site_id', '=', $site_id)
			->orderBy('name');

		return $this->findAll();
	}

	public function getTemplateFilePath()
	{
		return CMS_FOLDER . $this->_getDir() . '/template.htm';
	}

	public function saveTemplateFile($content)
	{
		$this->save();
		$this->_createDir();
		$content = trim($content);
		Core_File::write($this->getTemplateFilePath(), $content);
	}

	/**
	 * Save object.
	 *
	 * @return Core_Entity
	 */
	public function save()
	{
		$this->timestamp = Core_Date::timestamp2sql(time());
		return parent::save();
	}

	public function loadTemplateFile()
	{
		$path = $this->getTemplateFilePath();

		if (is_file($path))
		{
			return Core_File::read($path);
		}

		return NULL;
	}

	protected function _getDir()
	{
		return 'templates/template' . intval($this->id);
	}

	public function getTemplateCssFileHref()
	{
		return '/' . $this->_getDir() . '/style.css';
	}

	public function getTemplateCssFilePath()
	{
		return CMS_FOLDER . $this->_getDir() . '/style.css';
	}

	public function saveTemplateCssFile($content)
	{
		$this->save();
		$this->_createDir();
		$content = trim($content);
		Core_File::write($this->getTemplateCssFilePath(), $content);
	}

	public function loadTemplateCssFile()
	{
		$path = $this->getTemplateCssFilePath();

		if (is_file($path))
		{
			return Core_File::read($path);
		}

		return NULL;
	}

	protected function _createDir()
	{
		$sDirPath = dirname($this->getTemplateFilePath());

		if (!is_dir($sDirPath))
		{
			try
			{
				Core_File::mkdir($sDirPath, CHMOD, TRUE);
			} catch (Exception $e) {};
		}

		return $this;
	}

	/**
	 * Get parent comment
	 * @return Template_Model|NULL
	 */
	public function getParent()
	{
		if ($this->template_id)
		{
			return Core_Entity::factory('Template', $this->template_id);
		}
		return NULL;
	}

	// Используется при переносе шаблонов к макетам
	public function getByDataTemplateId($data_template_id)
	{
		$oTemplates = $this->Templates;
		$oTemplates->queryBuilder()
			//->clear()
			->where('data_template_id', '=', $data_template_id)
			->limit(1);

		$aTemplates = $oTemplates->findAll(FALSE);

		if (count($aTemplates) == 1)
		{
			return $aTemplates[0];
		}

		return NULL;
	}

	/**
	 * Delete object from database
	 * @param mixed $primaryKey primary key for deleting object
	 * @return Core_Entity
	 */
	public function delete($primaryKey = NULL)
	{
		if (is_null($primaryKey))
		{
			$primaryKey = $this->getPrimaryKey();
		}

		$this->id = $primaryKey;

		// Удаляем файл макета
		try
		{
			Core_File::delete($this->getTemplateFilePath());
		}
		catch (Exception $e) {}

		try
		{
			Core_File::delete($this->getTemplateCssFilePath());
		}
		catch (Exception $e) {}

		try
		{
			Core_File::deleteDir(CMS_FOLDER . $this->_getDir());
		}
		catch (Exception $e) {}

		$aTemplates = $this->Templates->findAll(FALSE);
		foreach($aTemplates as $oTemplate)
		{
			$oTemplate->delete();
		}

		return parent::delete($primaryKey);
	}

	/**
	 * Copy object
	 * @return Core_Entity
	 */
	public function copy()
	{
		$newObject = parent::copy();

		$newObject->saveTemplateCssFile($this->loadTemplateCssFile());
		$newObject->saveTemplateFile($this->loadTemplateFile());

		$aTemplates = $this->Templates->findAll();

		foreach ($aTemplates as $oTemplate)
		{
			$subTemplate = $oTemplate->copy();
			$subTemplate->template_id = $newObject->id;
			$subTemplate->save();
			//$newObject->add();
		}

		return $newObject;
	}
}