<?php

defined('HOSTCMS') || exit('HostCMS: access denied.');

/**
 * Updates.
 *
 * @package HostCMS 6\Update
 * @version 6.x
 * @author Hostmake LLC
 * @copyright © 2005-2012 ООО "Хостмэйк" (Hostmake LLC), http://www.hostcms.ru
 */
class Update_Dataset extends Admin_Form_Dataset
{
	protected $_count = NULL;

	public function getCount()
	{
		if (is_null($this->_count))
		{
			try
			{
				$this->_getUpdates();
			}
			catch (Exception $e)
			{
				Core_Message::show($e->getMessage(), 'error');
			}
		}

		return $this->_count;
	}

	protected $_objects = array();

	public function load()
	{
		return array_slice($this->_objects, $this->_offset, $this->_limit);
	}

	protected function _getUpdates()
	{
		$this->_objects = array();
		$this->_count = 0;

		$updateFilePath = Update_Controller::instance()->getFilePath();

		if (!file_exists($updateFilePath) || time() >= filemtime($updateFilePath) + 4 * 60 * 60)
		{
			if (!defined('HOSTCMS_USER_LOGIN') || !defined('HOSTCMS_CONTRACT_NUMBER') || !defined('HOSTCMS_PIN_CODE') || !defined('HOSTCMS_UPDATE_NUMBER'))
			{
				throw new Core_Exception(Core::_('Update.constant_check_error'), array(), 0, FALSE);
			}

			if (!defined('HOSTCMS_UPDATE_SERVER'))
			{
				throw new Core_Exception(Core::_('Update.update_constant_error'), array(), 0, FALSE);
			}

			Update_Controller::instance()
				->setUpdateOptions()
				->getUpdates();
		}

		// Дата окончания поддержки
		$error = 0;
		$expiration_of_support = FALSE;

		if (file_exists($updateFilePath))
		{
			$xml = Core_File::read($updateFilePath);

			$oXml = @simplexml_load_string($xml);

			if (is_object($oXml))
			{
				foreach ($oXml->update as $value)
				{
					$id = (int)$value->attributes()->id;

					$oUpdate_Entity = $this->_objects[$id] = $this->_newObject();
					$oUpdate_Entity->setTableColums(array(
						'id' => array(),
						'name' => array(),
						'description' => array(),
					));

					$oUpdate_Entity->id = $id;
					$oUpdate_Entity->name = html_entity_decode((string)$value->update_name, ENT_COMPAT, 'UTF-8');
					$oUpdate_Entity->description = html_entity_decode((string)$value->update_description, ENT_COMPAT, 'UTF-8');
				}

				$error = (int)$oXml->error;
				$expiration_of_support = (string)$oXml->expiration_of_support;
			}
		}
		else
		{
			throw new Core_Exception(Core::_('Update.error_open_updatefile'), array(), 0, FALSE);
		}

		$error = intval($error);

		if ($error > 0)
		{
			throw new Core_Exception(Core::_('Update.server_error_respond_' . $error), array(), 0, FALSE);
		}
		// Ошибок нет и количество обновления тоже 0
		elseif (count($this->_objects) == 0)
		{
			Core_Message::show(Core::_('Update.isLastUpdate'));
		}

		if ($expiration_of_support && !defined('IS_HOSTCMS_FREE'))
		{
			$f_expiration_of_support = Core_Date::sql2date($expiration_of_support);

			$this->_Admin_Form_Controller->addMessage(
				Core_Date::sql2timestamp($expiration_of_support) > time()
					? Core_Message::get(Core::_('Update.support_available', $f_expiration_of_support))
					: Core_Message::get(Core::_('Update.support_has_expired', $f_expiration_of_support, 'www.hostcms.ru'), 'error')
			);
		}

		$this->_count = count($this->_objects);

		return $this;
	}

	protected function _newObject()
	{
		return new Update_Entity();
	}

	public function getEntity()
	{
		return $this->_newObject();
	}

	public function getObject($primaryKey)
	{
		$primaryKey != 0 && !$this->_count && $this->_getUpdates();

		$return = isset($this->_objects[$primaryKey])
			? $this->_objects[$primaryKey]
			: $this->_newObject();

		if (isset($this->_objects[$primaryKey]))
		{
			unset($this->_objects[$primaryKey]);
		}

		return $return;
	}
}